# Script to calculate low, middle and high cloud fraction in GeoMIP model runs, save npy files, and make maps.
# Rick Russotto
# Started 30 June 2017

import numpy as np
import matplotlib.pyplot as plt
import netCDF4 as nc4
import geomipFunctions as gmf
from mpl_toolkits.basemap import Basemap
import os
from collections import OrderedDict


#Paths for the files (for CSIRO and HadGEM2-ES, load npy files from other scripts)
folder = '../nobackup/' 
#Cloud fractions
fpaths_cl_G1 = OrderedDict()
#fpaths_cl_G1['bn'] = folder+'cl_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc'
fpaths_cl_G1['bn'] = folder+'cl_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc'
fpaths_cl_G1['ca'] = folder+'cl_Amon_CanESM2_G1_r1i1p1_185001-194912.nc'
fpaths_cl_G1['cc'] = folder+'cl_Amon_CCSM4_G1_r1i1p1_185001-189912.nc'
fpaths_cl_G1['ce'] = folder+'cl_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc'
fpaths_cl_G1['gi'] = folder+'cl_Amon_GISS-E2-R_G1_r1i1p1_185001-191912.nc'
fpaths_cl_G1['ip'] = folder+'cl_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc'
fpaths_cl_G1['mi'] = folder+'cl_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc'
fpaths_cl_G1['mp'] = folder+'cl_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc' 
fpaths_cl_G1['no'] = folder+'cl_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc'

fpaths_cl_PI = OrderedDict()
fpaths_cl_PI['bn'] = folder+'cl_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc'
fpaths_cl_PI['ca'] = folder+'cl_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc'
fpaths_cl_PI['cc'] = folder+'cl_Amon_CCSM4_piControl_r1i1p1_080001-084912.nc'
fpaths_cl_PI['ce'] = folder+'cl_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc'
fpaths_cl_PI['gi'] = folder+'cl_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc'
fpaths_cl_PI['ip'] = folder+'cl_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-189912.nc'
fpaths_cl_PI['mi'] = folder+'cl_Amon_MIROC-ESM_piControl_r1i1p1_180001-185912.nc'
fpaths_cl_PI['mp'] = folder+'cl_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-189912.nc'
fpaths_cl_PI['no'] = folder+'cl_Amon_NorESM1-M_piControl_r1i1p1_070101-079912.nc'

fpaths_cl_A4 = OrderedDict()
fpaths_cl_A4['bn'] = folder+'cl_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_cl_A4['ca'] = folder+'cl_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_cl_A4['cc'] = folder+'cl_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-189912.nc'
fpaths_cl_A4['ce'] = folder+'cl_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc'
fpaths_cl_A4['gi'] = folder+'cl_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912.nc'
fpaths_cl_A4['ip'] = folder+'cl_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-194912.nc'
fpaths_cl_A4['mi'] = folder+'cl_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-006012.nc'
fpaths_cl_A4['mp'] = folder+'cl_Amon_MPI-ESM-LR_abrupt4xCO2_r1i1p1_185001-189912.nc'
fpaths_cl_A4['no'] = folder+'cl_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-010012.nc'


#Load NetCDF files & create NetCDF Dataset objects
Datasets_cl_G1 = OrderedDict()
for key, value in fpaths_cl_G1.items():
    print('Loading NetCDF file (cl): G1: ' + gmf.modelNames[key]) 
    Datasets_cl_G1[key] = nc4.Dataset(value)
    
Datasets_cl_PI = OrderedDict()    
for key, value in fpaths_cl_PI.items():
    print('Loading NetCDF file (cl): piControl: ' + gmf.modelNames[key]) 
    Datasets_cl_PI[key] = nc4.Dataset(value)    

Datasets_cl_A4 = OrderedDict()    
for key, value in fpaths_cl_A4.items():
    print('Loading NetCDF file (cl): abrupt4xCO2: ' + gmf.modelNames[key]) 
    Datasets_cl_A4[key] = nc4.Dataset(value)    
    

#First do interpolation to pressure coordinates; then calculate the low, middle and high cloud fractions assuming random overlap. 
#Save npy files in both cases. (Intermediate result may be useful for something else.)

modelsHybridSigma = ['bn', 'ca', 'cc', 'ce', 'cs', 'ip', 'mi', 'mp', 'no'] #Skip GISS for this--already in pressure coords. 

#For CSIRO and HadGEM2-ES, load the 4D npy files saved by other scripts
#(zonalMeanCloudFraction_CSIRO and zonalMeanCloudFraction_HadGEM2-ES)

#Whether to load npy files for the cloud fraction in 4 dimensions (time, pressure, lat, lon)
loadFromFile_4D_G1 ={'bn': False, 
                     'ca': False, 
                     'cc': False,
                     'ce': False, 
                     'cs': False,
                     'ip': False,
                     'mi': False, 
                     'mp': False, 
                     'no': False}

loadFromFile_4D_PI ={'bn': False, 
                     'ca': False, 
                     'cc': False,
                     'ce': False, 
                     'cs': False,
                     'ip': False,
                     'mi': False, 
                     'mp': False, 
                     'no': False}
                     
loadFromFile_4D_A4 ={'bn': False, 
                     'ca': False, 
                     'cc': False,
                     'ce': False, 
                     'cs': False,
                     'ip': False,
                     'mi': False, 
                     'mp': False, 
                     'no': False}
                     
#Interpolate the hybrid sigma output to pressure coordinates
levs = np.array([10, 20, 30, 50, 70, 100, 150, 200, 250, 300, 400, 500, 600, 700, 850, 925, 1000]) 

skipLoadPressureCoords = False #If already loading the final results (time averaged low, middle, high clouds) below, 
                               #make this true to save time loading the big netCDF files for clouds on pressure levels.
                               #Can also turn this flag off if just doing the HadGEM calculations. 

if not skipLoadPressureCoords:
    #G1: years 10-49 (years start from 0)
    cl_PressureCoords_G1 = dict() #Don't use orderedDicts anymore; can control plotting order using tuple of keys
    for key in modelsHybridSigma:
        if not loadFromFile_4D_G1[key]:
            print('Interpolating to pressure coordinates: G1: ' + gmf.modelNames[key])
            cl_PressureCoords_G1[key] = gmf.convertHybridSigmaToPressureCoords(Datasets_cl_G1[key], 'cl', levs, 10, 49)
            np.save('../nobackup/npyFiles/cl_4D_G1_'+key+'.npy', cl_PressureCoords_G1[key])
        else:
            print('Loading cloud fractions in pressure coordinates: G1: ' + gmf.modelNames[key])
            cl_PressureCoords_G1[key] = np.load('../nobackup/npyFiles/cl_4D_G1_'+key+'.npy')
            
    #piControl: years 0-39
    cl_PressureCoords_PI = dict() #Don't use orderedDicts anymore; can control plotting order using tuple of keys
    for key in modelsHybridSigma:
        if not loadFromFile_4D_PI[key]:
            print('Interpolating to pressure coordinates: piControl: ' + gmf.modelNames[key])
            cl_PressureCoords_PI[key] = gmf.convertHybridSigmaToPressureCoords(Datasets_cl_PI[key], 'cl', levs, 0, 39)
            np.save('../nobackup/npyFiles/cl_4D_PI_'+key+'.npy', cl_PressureCoords_PI[key])
        else:
            print('Loading cloud fractions in pressure coordinates: piControl: ' + gmf.modelNames[key])
            cl_PressureCoords_PI[key] = np.load('../nobackup/npyFiles/cl_4D_PI_'+key+'.npy')
            
    #abrupt4xCO2: years 10-49
    cl_PressureCoords_A4 = dict() #Don't use orderedDicts anymore; can control plotting order using tuple of keys
    for key in modelsHybridSigma:
        if not loadFromFile_4D_A4[key]:
            print('Interpolating to pressure coordinates: abrupt4xCO2: ' + gmf.modelNames[key])
            cl_PressureCoords_A4[key] = gmf.convertHybridSigmaToPressureCoords(Datasets_cl_A4[key], 'cl', levs, 10, 49)
            np.save('../nobackup/npyFiles/cl_4D_A4_'+key+'.npy', cl_PressureCoords_A4[key])
        else:
            print('Loading cloud fractions in pressure coordinates: abrupt4xCO2: ' + gmf.modelNames[key])
            cl_PressureCoords_A4[key] = np.load('../nobackup/npyFiles/cl_4D_A4_'+key+'.npy')
                
        
    # Insert the GISS output (already in pressure coordinates) into the "cl_PressureCoords" dicts. 
    # Subset in time as above, and first 17 pressure levels (giss output also includes 5, 3, 1, and .4 hPa which I don't need;
    # they are in order of descending pressure). 
    # Fliplr flipls along axis 1 (even for more than 2 dimensional arrays); this gets it into ascending order of pressure like the other models.
    cl_PressureCoords_G1['gi'] = np.fliplr(Datasets_cl_G1['gi'].variables['cl'][10*12:49*12+12, 0:17, :,:])
    cl_PressureCoords_PI['gi'] = np.fliplr(Datasets_cl_PI['gi'].variables['cl'][0:39*12+12, 0:17, :,:])
    cl_PressureCoords_A4['gi'] = np.fliplr(Datasets_cl_A4['gi'].variables['cl'][10*12:49*12+12, 0:17, :,:])

# Now do the overlap calculations to get the cloud fraction for low, middle and high clouds. 
modelsPressure = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'ip', 'mi', 'mp', 'no']

# If not plotting the annual mean results: set this flag to true
skipAnnual = False

if not skipAnnual:
    #Whether to load these from npy files (LMH stands for Low, Middle, High)
    loadFromFile_LMH_G1 ={'bn': False, 
                          'ca': False, 
                          'cc': False,
                          'ce': False, 
                          'cs': False,
                          'gi': False,
                          'ip': False,
                          'mi': False, 
                          'mp': False, 
                          'no': False}
    
    loadFromFile_LMH_PI ={'bn': False, 
                          'ca': False, 
                          'cc': False,
                          'ce': False,
                          'cs': False,
                          'gi': False,
                          'ip': False,
                          'mi': False, 
                          'mp': False, 
                          'no': False}
                         
    loadFromFile_LMH_A4 ={'bn': False, 
                          'ca': False, 
                          'cc': False,
                          'ce': False,
                          'cs': False,
                          'gi': False,
                          'ip': False,
                          'mi': False, 
                          'mp': False, 
                          'no': False}
                          
    loadFromFile_LMH_hg = False                 #Don't split by run for this model        
    
    cl_LMH_G1 = dict()
    for key in modelsPressure:
        if not loadFromFile_LMH_G1[key]:
            print('Doing cloud overlap calculations: G1: ' + gmf.modelNames[key])
            if(key == 'bn' or key == 'cc' or key == 'ce'): #Cloud fraction already between 0 and 1
                cl_LMH_G1[key] = gmf.calcLowMiddleHighClouds(cl_PressureCoords_G1[key], levs, divide100=False)
            else:
                cl_LMH_G1[key] = gmf.calcLowMiddleHighClouds(cl_PressureCoords_G1[key], levs, divide100=True)
            np.save('npyFiles/cl_LMH_G1_'+key+'.npy', cl_LMH_G1[key])
        else:
            print('Loading fractions of low, middle, and high clouds: G1: ' + gmf.modelNames[key])
            cl_LMH_G1[key] = np.load('npyFiles/cl_LMH_G1_'+key+'.npy').item()
    
    cl_LMH_PI = dict()
    for key in modelsPressure:
        if not loadFromFile_LMH_PI[key]:
            print('Doing cloud overlap calculations: piControl: ' + gmf.modelNames[key])
            if(key == 'bn' or key == 'cc' or key == 'ce'): #Cloud fraction already between 0 and 1
                cl_LMH_PI[key] = gmf.calcLowMiddleHighClouds(cl_PressureCoords_PI[key], levs, divide100=False)
            else:
                cl_LMH_PI[key] = gmf.calcLowMiddleHighClouds(cl_PressureCoords_PI[key], levs, divide100=True)
            np.save('npyFiles/cl_LMH_PI_'+key+'.npy', cl_LMH_PI[key])
        else:
            print('Loading fractions of low, middle, and high clouds: piControl: ' + gmf.modelNames[key])
            cl_LMH_PI[key] = np.load('npyFiles/cl_LMH_PI_'+key+'.npy').item()
            #print(cl_LMH_PI[key].keys()) #debug
    
    cl_LMH_A4 = dict()
    for key in modelsPressure:
        if not loadFromFile_LMH_A4[key]:
            print('Doing cloud overlap calculations: abrupt4xCO2: ' + gmf.modelNames[key])
            if(key == 'bn' or key == 'cc' or key == 'ce'): #Cloud fraction already between 0 and 1
                cl_LMH_A4[key] = gmf.calcLowMiddleHighClouds(cl_PressureCoords_A4[key], levs, divide100=False)
            else:
                cl_LMH_A4[key] = gmf.calcLowMiddleHighClouds(cl_PressureCoords_A4[key], levs, divide100=True)
            np.save('npyFiles/cl_LMH_A4_'+key+'.npy', cl_LMH_A4[key])
        else:
            print('Loading fractions of low, middle, and high clouds: abrupt4xCO2: ' + gmf.modelNames[key])
            cl_LMH_A4[key] = np.load('npyFiles/cl_LMH_A4_'+key+'.npy').item()
        
        
    #Calculate low, middle and high cloud fraction for HadGEM2-ES output which I've regridded to height coordinates:
    
    if not loadFromFile_LMH_hg:
        print('Loading regridded 4D cloud fraction output for HadGEM2-ES: piControl')
        cl_heightCoords_PI_hg = np.load('../nobackup/npyFiles/cl_4D_heightCoords_hg_PI.npy')
        print('Loading regridded 4D cloud fraction output for HadGEM2-ES: G1')
        cl_heightCoords_G1_hg = np.load('../nobackup/npyFiles/cl_4D_heightCoords_hg_G1.npy')
        print('Loading regridded 4D cloud fraction output for HadGEM2-ES: abrupt4xCO2')
        cl_heightCoords_A4_hg = np.load('../nobackup/npyFiles/cl_4D_heightCoords_hg_A4.npy')
        zLevels = np.flipud(np.array([25, 100, 300, 525, 750, 1000, 1500, 2250, 3000, 4000, 5000, 6000, 7000, 
                                      8500, 10000, 14000, 18000, 22000, 26000, 30000, 34000, 38000]))
        #zLevels = np.array([25, 100, 300, 525, 750, 1000, 1500, 2250, 3000, 4000, 5000, 6000, 7000, 
        #                              8500, 10000, 14000, 18000, 22000, 26000, 30000, 34000, 38000])
        print('Doing cloud overlap calculations: piControl: HadGEM2-ES')    
        cl_LMH_PI['hg'] = gmf.calcLowMiddleHighCloudsHeight(cl_heightCoords_PI_hg, zLevels, divide100=True)
        print('Doing cloud overlap calculations: G1: HadGEM2-ES')            
        cl_LMH_G1['hg'] = gmf.calcLowMiddleHighCloudsHeight(cl_heightCoords_G1_hg, zLevels, divide100=True)
        print('Doing cloud overlap calculations: abrupt4xCO2: HadGEM2-ES')    
        cl_LMH_A4['hg'] = gmf.calcLowMiddleHighCloudsHeight(cl_heightCoords_A4_hg, zLevels, divide100=True)
        np.save('npyFiles/cl_LMH_PI_hg.npy', cl_LMH_PI['hg'])            
        np.save('npyFiles/cl_LMH_G1_hg.npy', cl_LMH_G1['hg'])            
        np.save('npyFiles/cl_LMH_A4_hg.npy', cl_LMH_A4['hg'])            
    else:
        print('Loading fractions of low, middle and high clouds (HadGEM2-ES)')
        cl_LMH_PI['hg'] = np.load('npyFiles/cl_LMH_PI_hg.npy').item()
        cl_LMH_G1['hg'] = np.load('npyFiles/cl_LMH_G1_hg.npy').item()
        cl_LMH_A4['hg'] = np.load('npyFiles/cl_LMH_A4_hg.npy').item()
        
 


#####   GLOBAL MEAN CALCULATIONS   #####

#(These are in Table 2 of paper)

#(October 2017)

#Need to calculate the global means of changes in low, high clouds (might as well do middle too) for possible correlations with 
#cloud radiative effects/feedbacks and the deviation in the amount of solar reduction in G1 required from the formula based on albedo and 
#CO2 radiative forcing. 
#(Don't need seasonal versions of this)

#How do I calculate global mean changes? Have I done anything like this in GeoMIP?

#cl_LMH_G1[key][where] #um... the gmf.calcLowMiddleHighClouds function produces 2D arrays in lat, lon coordinates.
#For global mean need to weight by latitude. 

#Turns out I already did this in scriptUsingAPRPonGeoMIP. 
        
#Calculate the global mean of cloud fraction from the already time- averaged and overlap-calculated
#stuff saved in the "cl_LMH" arrays.
#dictName: which "cl_LMH" dict above to use
#where: string 'low', 'middle', 'high' 
#key: which model
        
lats = np.load('npyFiles/modelLats.npy').item()
def calcGlobalMeanCloudFraction(dictName, where, key):
    Data = dictName[key][where]
    #lat = Datasets_cl_PI[key].variables['lat'][:]
    lat = lats[key]
    areaWeights = np.cos(lat*np.pi/180)
    areaWeights2D = np.tile(areaWeights,(np.shape(Data)[1],1))
    #print(np.shape(areaWeights2D)) #debug
    areaWeights2D = np.swapaxes(areaWeights2D,0,1)
    weighted2DMatrix = Data*areaWeights2D
    sumWeighted = np.sum(weighted2DMatrix)
    sumWeights = np.sum(areaWeights2D)
    weightedMean = sumWeighted/sumWeights
    return weightedMean
    
skipGlobalMeans = False

if not skipGlobalMeans:
        
    #Do this for each run, and type of cloud, saving in dicts; save dicts to npy files; then print differences and save them in the comments here.
    globalMean_cl_PI_low = OrderedDict()
    globalMean_cl_PI_middle = OrderedDict()
    globalMean_cl_PI_high = OrderedDict()
    globalMean_cl_G1_low = OrderedDict()
    globalMean_cl_G1_middle = OrderedDict()
    globalMean_cl_G1_high = OrderedDict()
    globalMean_cl_A4_low = OrderedDict()
    globalMean_cl_A4_middle = OrderedDict()
    globalMean_cl_A4_high = OrderedDict()
    
    keys = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']
    for key in keys:
        globalMean_cl_PI_low[key] = calcGlobalMeanCloudFraction(cl_LMH_PI, 'low', key)
        globalMean_cl_PI_middle[key] = calcGlobalMeanCloudFraction(cl_LMH_PI, 'middle', key)
        globalMean_cl_PI_high[key] = calcGlobalMeanCloudFraction(cl_LMH_PI, 'high', key)
        globalMean_cl_G1_low[key] = calcGlobalMeanCloudFraction(cl_LMH_G1, 'low', key)
        globalMean_cl_G1_middle[key] = calcGlobalMeanCloudFraction(cl_LMH_G1, 'middle', key)
        globalMean_cl_G1_high[key] = calcGlobalMeanCloudFraction(cl_LMH_G1, 'high', key)
        globalMean_cl_A4_low[key] = calcGlobalMeanCloudFraction(cl_LMH_A4, 'low', key)
        globalMean_cl_A4_middle[key] = calcGlobalMeanCloudFraction(cl_LMH_A4, 'middle', key)
        globalMean_cl_A4_high[key] = calcGlobalMeanCloudFraction(cl_LMH_A4, 'high', key)
    
    np.save('npyFiles/globalMean_cl_PI_low.npy', globalMean_cl_PI_low)
    np.save('npyFiles/globalMean_cl_PI_middle.npy', globalMean_cl_PI_middle)
    np.save('npyFiles/globalMean_cl_PI_high.npy', globalMean_cl_PI_high)
    np.save('npyFiles/globalMean_cl_G1_low.npy', globalMean_cl_G1_low)
    np.save('npyFiles/globalMean_cl_G1_middle.npy', globalMean_cl_G1_middle)
    np.save('npyFiles/globalMean_cl_G1_high.npy', globalMean_cl_G1_high)
    np.save('npyFiles/globalMean_cl_A4_low.npy', globalMean_cl_A4_low)
    np.save('npyFiles/globalMean_cl_A4_middle.npy', globalMean_cl_A4_middle)
    np.save('npyFiles/globalMean_cl_A4_high.npy', globalMean_cl_A4_high)
    
    print('Global mean change in cloud fraction:')
    print('G1 minus piControl:')
    print('Low clouds:')
    for key in keys:
        print(key + ': ' + str(globalMean_cl_G1_low[key] - globalMean_cl_PI_low[key]))
    print('Middle clouds:')
    for key in keys:
        print(key + ': ' + str(globalMean_cl_G1_middle[key] - globalMean_cl_PI_middle[key]))
    print('High clouds:')    
    for key in keys:
        print(key + ': ' + str(globalMean_cl_G1_high[key] - globalMean_cl_PI_high[key]))
    print('abrupt4xCO2 minus piControl:')
    print('Low clouds:')
    for key in keys:
        print(key + ': ' + str(globalMean_cl_A4_low[key] - globalMean_cl_PI_low[key]))
    print('Middle clouds:')
    for key in keys:
        print(key + ': ' + str(globalMean_cl_A4_middle[key] - globalMean_cl_PI_middle[key]))
    print('High clouds:')    
    for key in keys:
        print(key + ': ' + str(globalMean_cl_A4_high[key] - globalMean_cl_PI_high[key]))
    
    #Console output (March 8, 2018, after fixes for BNU, CCSM, CESM, GISS, and adding CSIRO and HadGEM2-ES)
    #These are in fraction, not percent.
    #Global mean change in cloud fraction:
    #G1 minus piControl:
    #Low clouds:
    #bn: -0.00596628083095
    #ca: -0.0158893007223
    #cc: -0.0154416665132
    #ce: -0.0150540619341
    #cs: -0.0070655508274
    #gi: -0.010393573853
    #hg: -0.0138130015808
    #ip: -0.00743595858916
    #mi: -0.0159524993752
    #mp: -0.0103275406447
    #no: -0.0163380092607
    #Middle clouds:
    #bn: 0.00261187966707
    #ca: -0.00135339094001
    #cc: -0.00153564847293
    #ce: 0.000296826094828
    #cs: -0.00514136174833
    #gi: 0.0012636381387
    #hg: -0.00185269044652
    #ip: 0.000334169959653
    #mi: -0.000151209065941
    #mp: -0.000470919107206
    #no: -0.00216905682666
    #High clouds:
    #bn: 0.00912337262571
    #ca: 0.00685734914569
    #cc: 0.0119261010633
    #ce: 0.0138179571246
    #cs: -0.00566496994727
    #gi: 0.0133560010483
    #hg: 0.00435216794561
    #ip: -0.011176186296
    #mi: 0.00764208782781
    #mp: 0.0131021098053
    #no: 0.0112072910539
    #abrupt4xCO2 minus piControl:
    #Low clouds:
    #bn: 0.000399459438904
    #ca: -0.0230758003702
    #cc: -0.00255234196641
    #ce: -0.0308933123861
    #cs: -0.013236569967
    #gi: -0.0082732155132
    #hg: -0.0319160766284
    #ip: -0.0369725412827
    #mi: -0.0213411718022
    #mp: -0.0231810598024
    #no: -0.00518005198883
    #Middle clouds:
    #bn: -0.00694921673777
    #ca: -0.0169816070041
    #cc: -0.010453344592
    #ce: -0.0337207134965
    #cs: -0.0192207774866
    #gi: -0.00588542033915
    #hg: -0.0424963814637
    #ip: -0.0305157577584
    #mi: -0.0198833134077
    #mp: -0.0340728872117
    #no: -0.011935300993
    #High clouds:
    #bn: 0.0124272481011
    #ca: 0.0179384552167
    #cc: 0.00953074259598
    #ce: -0.000191418980863
    #cs: -0.0104154130303
    #gi: 0.0153919521248
    #hg: 0.0145594236944
    #ip: -0.0214690869452
    #mi: 0.0129106019876
    #mp: -0.000832754874906
    #no: 0.00342168405988


#For plots: model lats and lons
modelLats = np.load('npyFiles/modelLats.npy').item()
modelLons = np.load('npyFiles/modelLons.npy').item()  

#Flag to skip entire plotting section (useful for global mean part)
skipPlots = False

if not skipPlots:    
        
    #####   PLOTS   #####

    
    #Difference plots between runs
    #key: model 2-digit code
    #run: G1, PI, A4; run1 - run2
    #where: "low", "middle", "high"

            
    #March 2018: 
    #PNG versions with smaller colorbar and tight_layout
    #(for individual models in Supplemental Information for paper)
    def mapDiffContourfPNG(figNo, key, run1, run2, where, clevs=np.linspace(-10, 10, 21), save=False):
        w,h = plt.figaspect(1./2.5)  
        plt.figure(figNo, figsize=(w,h))            
        lat = modelLats[key]
        lon = modelLons[key]
        if(run1 == 'G1' and run2 == 'PI'):
            plotData = cl_LMH_G1[key][where] - cl_LMH_PI[key][where]
        elif(run1 == 'A4' and run2 == 'PI'):
            plotData = cl_LMH_A4[key][where] - cl_LMH_PI[key][where]
        else:
            print('Haven\'t programmed this combination of runs yet; exiting plot difference function')
            return
        titleString = gmf.modelNames[key]
        #Duplicate edge data to eliminate white strip on left side
        lon = np.append(lon,360.) 
        plotData = np.append(plotData, plotData[:,0:1], axis=1)  
        plotData = plotData * 100. #Convert from fraction to percent
        #Make map
        Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
        Map.drawcoastlines(linewidth=1)
        Map.drawparallels(np.linspace(-90.,90.,7))
        Map.drawmeridians(np.linspace(-150., 150., 11))
        lons, lats = np.meshgrid(lon, lat)
        #thePlot = Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu, levels=clevs, extend='both')
        thePlot = Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu_r, levels=clevs, extend='both')
        cbar = plt.colorbar(thePlot)
        cbar.ax.set_ylabel('Cloud fraction difference (%)', fontsize=24)
        cbar.ax.tick_params(labelsize=22)
        plt.title(titleString, fontsize=26)
        plt.tight_layout()
        if(save):   
            #plt.savefig('plots/individualModels/cl_map_'+run1+'-'+run2+'_'+key+'_'+where+'.png') 
            plt.savefig('plots/individualModels/cl_map_'+run1+'-'+run2+'_'+key+'_'+where+'_flipped.png') 
    
    
    
    ###G1 minus piControl
    mapDiffContourfPNG(430, 'bn', 'G1', 'PI', 'low', save=True)
    mapDiffContourfPNG(431, 'ca', 'G1', 'PI', 'low', save=True)
    mapDiffContourfPNG(432, 'cc', 'G1', 'PI', 'low', save=True)
    mapDiffContourfPNG(433, 'ce', 'G1', 'PI', 'low', save=True)
    mapDiffContourfPNG(434, 'cs', 'G1', 'PI', 'low', save=True)
    mapDiffContourfPNG(435, 'gi', 'G1', 'PI', 'low', save=True)
    mapDiffContourfPNG(436, 'hg', 'G1', 'PI', 'low', save=True)            
    mapDiffContourfPNG(437, 'ip', 'G1', 'PI', 'low', save=True)
    mapDiffContourfPNG(438, 'mi', 'G1', 'PI', 'low', save=True)
    mapDiffContourfPNG(439, 'mp', 'G1', 'PI', 'low', save=True)
    mapDiffContourfPNG(440, 'no', 'G1', 'PI', 'low', save=True)
    
           
    mapDiffContourfPNG(540, 'bn', 'G1', 'PI', 'middle', save=True)
    mapDiffContourfPNG(541, 'ca', 'G1', 'PI', 'middle', save=True)        
    mapDiffContourfPNG(542, 'cc', 'G1', 'PI', 'middle', save=True)        
    mapDiffContourfPNG(543, 'ce', 'G1', 'PI', 'middle', save=True)        
    mapDiffContourfPNG(544, 'cs', 'G1', 'PI', 'middle', save=True)        
    mapDiffContourfPNG(545, 'gi', 'G1', 'PI', 'middle', save=True)        
    mapDiffContourfPNG(546, 'hg', 'G1', 'PI', 'middle', save=True)        
    mapDiffContourfPNG(547, 'ip', 'G1', 'PI', 'middle', save=True)        
    mapDiffContourfPNG(548, 'mi', 'G1', 'PI', 'middle', save=True)        
    mapDiffContourfPNG(549, 'mp', 'G1', 'PI', 'middle', save=True)        
    mapDiffContourfPNG(550, 'no', 'G1', 'PI', 'middle', save=True)       
            
    mapDiffContourfPNG(650, 'bn', 'G1', 'PI', 'high', save=True)
    mapDiffContourfPNG(651, 'ca', 'G1', 'PI', 'high', save=True)                
    mapDiffContourfPNG(652, 'cc', 'G1', 'PI', 'high', save=True)                
    mapDiffContourfPNG(653, 'ce', 'G1', 'PI', 'high', save=True)                
    mapDiffContourfPNG(654, 'cs', 'G1', 'PI', 'high', save=True)           
    mapDiffContourfPNG(655, 'gi', 'G1', 'PI', 'high', save=True)                
    mapDiffContourfPNG(656, 'hg', 'G1', 'PI', 'high', save=True)                
    mapDiffContourfPNG(657, 'ip', 'G1', 'PI', 'high', save=True)                
    mapDiffContourfPNG(658, 'mi', 'G1', 'PI', 'high', save=True)                
    mapDiffContourfPNG(659, 'mp', 'G1', 'PI', 'high', save=True)                
    mapDiffContourfPNG(660, 'no', 'G1', 'PI', 'high', save=True)    
    
    #plt.show()
    