# Script to test hypothesis that percentage solar reduction in G1 correlates 
# with model's equilibrium climate sensitivity

import numpy as np
import matplotlib.pyplot as plt
from collections import OrderedDict

#Solar constant reductions (percent)
#(Source: Kravitz et al. 2013 G1 overview paper, except for CSIRO which I calculated myself)
reductions = {'bn': 4.4, #Updated Dec. 3rd, 2017 for r3i1p1 simulation
              'ca': 4.0, 
              'cc': 4.1, 
              'ce': 4.7, 
              'cs': 3.2,
              'ec': 4.3, 
              'gi': 4.5, 
              'hc': 4.1, 
              'hg': 3.9, 
              'ip': 3.5, 
              'mi': 5.0, 
              'mp': 4.7, 
              'no': 4.0}

#Equilibrium climate sensitivity (K warming for 2xCO2)
#Sources: 
#A12 = Andrews et al. (2012) Geophysical Research Letters: 
#      Forcing, feedbacks and climate sensitivity in CMIP5 coupled atmosphere-ocean climate models
sensitivities = {#'bn': ,             
                 'ca': 3.69, #A12
                 #'cc': , 
                 #'ce': , 
                 #'cs': ,
                 #'ec': , 
                 #'gi': , 
                 #'hc': , 
                 'hg': 4.59, #A12
                 'ip': 4.13, #A12
                 'mi': 4.67, #A12
                 'mp': 3.63, #A12
                 'no': 2.80} #A12
                 
#Actually, probably shouldn't mix and match different sources because methodologies differ and there are slightly different values for ECS. 
#Sherwood et al. (Nature, 2014; S14) has all 6 of the above models, plus BNU, CCSM4, and GISS-E2-R. This is also based on abrupt4xCO2, 
#better for comparison with G1 than...actually Andrews et al. also used abrupt4xCO2. 
           
sensitivitiesS14 = {'bn': 4.11,             
                    'ca': 3.68, 
                    'cc': 2.92, 
                    #'ce': , 
                    #'cs': ,
                    #'ec': , 
                    'gi': 2.11, 
                    #'hc': , 
                    'hg': 4.55, 
                    'ip': 4.10, 
                    'mi': 4.65, 
                    'mp': 3.60, 
                    'no': 2.83} 

#Array version of this for calculations
sensitivitiesS14array = np.array([4.11, 3.68, 2.92, 2.11, 4.55, 4.10, 4.65, 3.60, 2.83])

                    
#Additional calculation: plot predicted based on forcing/albedo formula vs. actual solar reduction
#Radiative forcing for a doubling of CO2, from S14 (W m^-2)

forcingsS14 = {'bn': 3.78, 
               'ca': 3.85,
               'cc': 3.70, 
               #'ce': , 
               #'cs': , 
               #'ec': , 
               'gi': 3.77, 
               #'hc': ,
               'hg': 2.95, 
               'ip': 3.12, 
               'mi': 4.27, 
               'mp': 4.15,
               'no': 3.21}                    
               
#Array version of this for calculations
forcingsS14array = np.array([3.78, 3.85, 3.70, 3.77, 2.95, 3.12, 4.27, 4.15, 3.21 ])

#Planetary albedo (%) (piContol), from Kravitz et al. 2013 (K13) (note planetary albedo is different in abrupt4xCO2, not sure which one modelers used)

albedosK13 =  {'bn': 29.8, 
               'ca': 29.5,
               'cc': 28.7, 
               #'ce': , 
               #'cs': , 
               #'ec': , 
               'gi': 29.5, 
               #'hc': ,
               'hg': 28.7, 
               'ip': 30.7, 
               'mi': 31.3, 
               'mp': 30.2,
               'no': 30.8}                    
               
#Array version
albedosK13array = np.array([29.8, 29.5, 28.7, 29.5, 28.7, 30.7, 31.3, 30.2, 30.8])

#Predicted solar constant change based on formula: (delta S0) = 4*(delta RF)/(1-alpha) (W/m^2)

predictedDeltaS0array = 2*4.*forcingsS14array/(1.-albedosK13array/100.)  #2* for quadrupling instead of doubling of CO2

#Need dict version of this for plot
predictedDeltaS0 = dict()
for key in albedosK13.keys():
    predictedDeltaS0[key] = 2*4.*forcingsS14[key]/(1.-albedosK13[key]/100.)

#Calculate the forcing associated with the solar reduction, in W/m^2 (halved for CO2 doubling instead of quadrupling)
solarForcings = OrderedDict()
for key in ['bn', 'ca', 'cc', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']:
    solarForcings[key] =  (reductions[key]/100.)*1361./4.*(1.-albedosK13[key]/100.)/2.

solarForcingsArray = solarForcings.values() 








#Scatter plots
            
#A12 subset: 
plt.figure(0)
plt.scatter([3.69, 4.59, 4.13, 4.67, 3.63, 2.80], [4.0, 3.9, 3.5, 5.0, 4.7, 4.0])
plt.xlabel('ECS (K)')
plt.ylabel('Solar constant reduction in G1 (%)')
plt.title('Andrews et al. (2012) model subset\n r = '+
          str(np.around(
              np.corrcoef([3.69, 4.59, 4.13, 4.67, 3.63, 2.80], [4.0, 3.9, 3.5, 5.0, 4.7, 4.0])[0,1],3)))


#S14 subset:
plt.figure(1)
plt.scatter([4.11, 3.68, 2.92, 2.11, 4.55, 4.10, 4.65, 3.60, 2.83], [4.4, 4.0, 4.1, 4.5, 3.9, 3.5, 5.0, 4.7, 4.0])
plt.xlabel('ECS (K)')
plt.ylabel('Solar constant reduction in G1 (%)')
plt.title('Sherwood et al. (2014) model subset\n r = '+
          str(np.around(
              np.corrcoef([4.11, 3.68, 2.92, 2.11, 4.55, 4.10, 4.65, 3.60, 2.83], [4.4, 4.0, 4.1, 4.5, 3.9, 3.5, 5.0, 4.7, 4.0])[0,1],3)))
              
#S14 subset excluding BNU-ESM: 
plt.figure(2)
plt.scatter([3.68, 2.92, 2.11, 4.55, 4.10, 4.65, 3.60, 2.83], [4.0, 4.1, 4.5, 3.9, 3.5, 5.0, 4.7, 4.0])
plt.xlabel('ECS (K)')
plt.ylabel('Solar constant reduction in G1 (%)')
plt.title('Sherwood et al. (2014) model subset (no BNU-ESM)\n r = '+
          str(np.around(
              np.corrcoef([3.68, 2.92, 2.11, 4.55, 4.10, 4.65, 3.60, 2.83], [4.0, 4.1, 4.5, 3.9, 3.5, 5.0, 4.7, 4.0])[0,1],3)))


#Okay, make a version of Figure 1 that has color coded dots for models, and a legend. 

#Dictionary of colors for each model for plotting purposes
#Note the 2-digit codes for the model which I will consistently use as dictionary keys. 
modelColors = OrderedDict()
modelColors['bn'] = 'magenta'          #BNU-ESM
modelColors['ca'] = 'indigo'           #CanESM-2
modelColors['cc'] = 'cornflowerblue'   #CCSM4
modelColors['ce'] = 'blue'             #CESM-CAM5.1-FV
modelColors['cs'] = 'teal'             #CSIRO
modelColors['ec'] = 'springgreen'      #EC-EARTH-DMI
modelColors['gi'] = 'green'            #GISS-E2-R
modelColors['hc'] = 'lime'             #HadCM3
modelColors['hg'] = 'gold'             #HadGEM2-ES
modelColors['ip'] = 'darkgoldenrod'    #IPSL-CM5A-LR
modelColors['mi'] = 'orange'           #MIROC-ESM
modelColors['mp'] = 'red'              #MPI-ESM-LR
modelColors['no'] = 'maroon'           #NorESM1
modelColors['mm'] = 'k'                #Multi-Model Mean 

#On that note, need dictionary of names for the models, for plot legends. 
modelNames = OrderedDict()
modelNames['bn'] = 'BNU-ESM'
modelNames['ca'] = 'CanESM-2'
modelNames['cc'] = 'CCSM4'
modelNames['ce'] = 'CESM-CAM5.1-FV'
modelNames['cs'] = 'CSIRO-Mk3L-1-2'
modelNames['ec'] = 'EC-EARTH-DMI'
modelNames['gi'] = 'GISS-E2-R'
modelNames['hc'] = 'HadCM3'
modelNames['hg'] = 'HadGEM2-ES'
modelNames['ip'] = 'IPSL-CM5A-LR'
modelNames['mi'] = 'MIROC-ESM'
modelNames['mp'] = 'MPI-ESM-LR'
modelNames['no'] = 'NorESM1'
modelNames['mm'] = 'Multi-Model Mean'    

(w,h) = plt.figaspect(1)
plt.figure(3, figsize=(w,h))
legendEntries = OrderedDict()
legendText = OrderedDict()
for key in ['bn', 'ca', 'cc', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']:
#for key in ['ca', 'cc', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']: #Skip (old) BNU 
    legendEntries[key] = plt.scatter(sensitivitiesS14[key], reductions[key], 
                                     s=150, c=modelColors[key])
    legendText[key] = modelNames[key]
plt.xlabel('Equilibrium climate sensitivity (K)', fontsize=15)
plt.ylabel('Solar constant reduction in G1 (%)', fontsize=15) 
plt.title('r = '+
          str(np.around(
              np.corrcoef([4.11, 3.68, 2.92, 2.11, 4.55, 4.10, 4.65, 3.60, 2.83], [4.4, 4.0, 4.1, 4.5, 3.9, 3.5, 5.0, 4.7, 4.0])[0,1],2)), fontsize=15)
plt.tick_params(axis='both', which='major', labelsize=13)              
#plt.legend(legendEntries.values(), legendText.values(), frameon=True, loc='lower left',scatterpoints=1, prop={'size': 13})
#plt.savefig('plots/solarReductionVsECS.pdf')



#Plot of predicted vs. actual S0 reduction
plt.figure(4)
legendEntries = OrderedDict()
legendText = OrderedDict()
for key in ['bn', 'ca', 'cc', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']:
    legendEntries[key] = plt.scatter(predictedDeltaS0[key], reductions[key], 
                                     s=150, c=modelColors[key])
    legendText[key] = modelNames[key]
plt.xlabel('Predicted reduction in solar constant (W m$^{-2}$)')
plt.ylabel('Solar constant reduction in G1 (%)') 
plt.title('r = '+str(np.around(np.corrcoef(predictedDeltaS0array,  [4.4, 4.0, 4.1, 4.5, 3.9, 3.5, 5.0, 4.7, 4.0] )[0,1],3)))
plt.legend(legendEntries.values(), legendText.values(), frameon=True, loc='upper left',scatterpoints=1, prop={'size': 11})
#plt.savefig('plots/predictedVsActualSolarReduction.pdf')

#Version with predicted percent solar reduction, and 1:1 line
plt.figure(5, figsize=plt.figaspect(1))
legendEntries = OrderedDict()
legendText = OrderedDict()
for key in ['bn', 'ca', 'cc', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']:
#for key in ['ca', 'cc', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']:
    legendEntries[key] = plt.scatter(predictedDeltaS0[key]/1361.*100., reductions[key], 
                                     s=150, c=modelColors[key])
    legendText[key] = modelNames[key]
plt.axis([2, 5.5, 2, 5.5])
plt.plot([2, 5.5], [2, 5.5], 'k')    
plt.xlabel('Predicted reduction in solar constant (%)', fontsize=15)
plt.ylabel('Solar constant reduction in G1 (%)', fontsize=15) 
plt.title('r = '+str(np.around(np.corrcoef(predictedDeltaS0array,  [4.4, 4.0, 4.1, 4.5, 3.9, 3.5, 5.0, 4.7, 4.0] )[0,1],2)), fontsize=15)
plt.legend(legendEntries.values(), legendText.values(), frameon=True, loc='lower right',scatterpoints=1, prop={'size': 13})
plt.tick_params(axis='both', which='major', labelsize=13)
plt.gca().set_aspect(1)
#plt.savefig('plots/predictedVsActualSolarReductionPercent.pdf')



#F2x vs. ECS from S14
plt.figure(6)
legendEntries = OrderedDict()
legendText = OrderedDict()
for key in ['bn', 'ca', 'cc', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']:
    legendEntries[key] = plt.scatter(forcingsS14[key], sensitivitiesS14[key], s=150, c=modelColors[key])
    legendText[key] = modelNames[key]
plt.xlabel('Radiative forcing for 2xCO$_2$ (W m$^{-2}$)')
plt.ylabel('Equilibrium climate sensitivity (K)')
plt.title('r = '+str(np.around(np.corrcoef(forcingsS14array, sensitivitiesS14array)[0,1],3)))
plt.legend(legendEntries.values(), legendText.values(), frameon=True, loc='lower right', scatterpoints=1, prop={'size':11})
#plt.savefig('plots/forcingVsECS_2xCO2.pdf')

#Solar forcing vs CO2 forcing
#(But I'm not sure I can calculate adjustments to solar forcing)
#(This looks very similar to Figure 5, but this isn't really solar foring--can't calculate it including tropospheric adjustment.)

plt.figure(7)
legendEntries = OrderedDict()
legendText = OrderedDict()
for key in ['bn', 'ca', 'cc', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']:
    legendEntries[key] = plt.scatter(forcingsS14[key], solarForcings[key], s=150, c=modelColors[key])
    legendText[key] = modelNames[key]
plt.xlabel('Radiative forcing for 2xCO$_2$ (W m$^{-2}$)')
plt.ylabel('Radiative forcing of solar reduction (W m$^{-2}$)')
plt.title('r = '+str(np.around(np.corrcoef(forcingsS14array, solarForcingsArray)[0,1],3)))
plt.legend(legendEntries.values(), legendText.values(), frameon=True, loc='lower right', scatterpoints=1, prop={'size':11})
#plt.savefig('plots/CO2vsSolarForcing.pdf')

#plt.show()

#January 30, 2018: Instantaneous forcing version


#Instantaneous forcings from Zhang and Huang, 2014, Table 1
#(These are for CO2 quadrupling, not doubling)
#Not a lot of the same models--ask 
instaForcingsZH = {'ca': 5.74, 
                   'cc': 5.51, 
                   'hg': 5.55,
                   'ip': 4.51,
                   'no': 5.55}
                   
#Calculate the predicted delta s0 from these

#Print the predicted delta s0
#Need dict version of this for plot
predictedDeltaS0_InstaZH = dict()
predictedSolarReduction_InstaZH = dict() #This one in percentage terms
residuals_InstaZH = dict()
residuals_wm2_InstaZH = dict()
modelsZH = ['ca', 'cc', 'hg', 'ip', 'no']
for key in modelsZH:
    predictedDeltaS0_InstaZH[key] = 4.*instaForcingsZH[key]/(1.-albedosK13[key]/100.) #Don't multiply by 2 this time because forcing was for a quadrupling
    predictedSolarReduction_InstaZH[key] = predictedDeltaS0_InstaZH[key]/1361.*100. #In percentage terms
    residuals_InstaZH[key] = reductions[key] - predictedSolarReduction_InstaZH[key]
    residuals_wm2_InstaZH[key] = residuals_InstaZH[key]/100.*1361./4.*(1.-albedosK13[key]/100.)
    print(key)
    print(predictedDeltaS0_InstaZH[key])
    print(predictedSolarReduction_InstaZH[key])
    print(residuals_InstaZH[key])
    print(residuals_wm2_InstaZH[key])


#Console output
#ca
#32.5673758865
#2.39290050599
#1.60709949401
#3.85505
#cc
#30.9116409537
#2.27124474311
#1.82875525689
#4.43652825
#hg
#31.1360448808
#2.28773290821
#1.61226709179
#3.91133175
#ip
#26.0317460317
#1.91269258132
#1.58730741868
#3.74276375
#no
#32.0809248555
#2.35715832884
#1.64284167116
#3.86812

#The bottom lines match very well with the rightmost bar on the most recent collection of bar graphs. Success!




