# Script to make maps of changes in LW cloud radiative effect in the GeoMIP ensemble
#
# Rick Russotto
# Started 30 October 2017
#
# 21 January 2018: correct LW CRE change for cloud masking effects
#
# 19 February 2018: add BNU model (r3i1p1)
# 20 February 2018: add fixed version of GISS rlutcs, with masked-out missing values

import numpy as np
import matplotlib.pyplot as plt
import netCDF4 as nc4
import geomipFunctions as gmf
import os
from mpl_toolkits.basemap import Basemap
from collections import OrderedDict

    
#LW cloud radiative effect is all sky minus clear sky net downward at TOA. Same as clear sky minus all sky net upward, so 'rlutcs' - 'rlut'. 
#So all I need to do is load rlutcs and rlut, take time mean and save to npy files. Then take global means, and plot difference maps. 

#OK: first need the file names for the NetCDF files. Copy dicts from "runEBM.py"; delete the non-rlut variables, 
#    add any additional models for which I can do this calculation.
    
folder='../nobackup/'
folderCCSM = '../nobackup/CCSM4_fluxes_from_NCAR/' #For corrected rlut output
folderCESM = '../nobackup/CESM_corrected/'
folderNor = '../nobackup/NorESM_corrected_rlut/'

#See notes on the correction factor for rlutcs which I added manually, after downloading corrected versions of rlut for 
#CCSM4 and NorESM and doing the correction for CESM myself. The corrections are saved by "correctCESM_rlut.py". I previously
#dealt with this in the "runEBM.py" script. Also the GISS output had flipped sign of clear-sky OLR (rlutcs).

#In addition to the models in "runEBM.py", can do CSIRO. (Didn't actually run EBM for GISS because of problem with rsuscs; similar for BNU.)

fluxPaths_PI = OrderedDict()
fluxPaths_PI['bn'] = {'rlut': folder+'rlut_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      'rlutcs': folder+'rlutcs_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      }
fluxPaths_PI['ca'] = {'rlut': folder+'rlut_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc',
                      'rlutcs': folder+'rlutcs_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc',
                      }
fluxPaths_PI['cc'] = {'rlut': folderCCSM+'rlut_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc',     #Need to add correction for rlutcs--
                      'rlutcs': folderCCSM+'rlutcs_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc', #saved by "correctCESM_rlut.py" (despite rlutcs being in this folder)
                      }
fluxPaths_PI['ce'] = {'rlut': folderCESM+'rlut_Amon_CESM1-CAM5.1-FV2_piControl_r1i1p1_000101-005012.nc', 
                      'rlutcs': folder+'rlutcs_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc', #Need to add correction for rlutcs--
                      }                                                                                  #saved by "correctCESM_rlut.py"
fluxPaths_PI['cs'] = {'rlut': folder+'rlut_Amon_CSIRO-Mk3L-1-2_piControl_r1i1p1_000101-100012.nc', 
                      'rlutcs': folder+'rlutcs_Amon_CSIRO-Mk3L-1-2_piControl_r1i1p1_000101-100012.nc',
                      }                      
fluxPaths_PI['gi'] = {'rlut': folder+'rlut_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc', 
                      'rlutcs': folder+'rlutcs_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912_fixed.nc',
                      }   
fluxPaths_PI['hg'] = {'rlut': folder+'rlut_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc',   
                      'rlutcs': folder+'rlutcs_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc',
                      }     
fluxPaths_PI['ip'] = {'rlut': folder+'rlut_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc',   
                      'rlutcs': folder+'rlutcs_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc',
                      }      
fluxPaths_PI['mi'] = {'rlut': folder+'rlut_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc', 
                      'rlutcs': folder+'rlutcs_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc',
                      }
fluxPaths_PI['mp'] = {'rlut': folder+'rlut_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-203512.nc',   
                      'rlutcs': folder+'rlutcs_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-203512.nc',
                      }                                            
fluxPaths_PI['no'] = {'rlut': folderNor+'rlut_Amon_NorESM1-M_piControl_r1i1p1_070101-120012.nc',  
                      'rlutcs': folder+'rlutcs_Amon_NorESM1-M_piControl_r1i1p1_070001-115012.nc',        #Need to add correction for rlutcs--
                      }                                                                                  #saved by "correctCESM_rlut.py"
                      

fluxPaths_G1 = OrderedDict()
fluxPaths_G1['bn'] = {'rlut': folder+'rlut_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      'rlutcs': folder+'rlutcs_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      }
fluxPaths_G1['ca'] = {'rlut': folder+'rlut_Amon_CanESM2_G1_r1i1p1_185001-194912.nc',
                      'rlutcs': folder+'rlutcs_Amon_CanESM2_G1_r1i1p1_185001-194912.nc',
                      }
fluxPaths_G1['cc'] = {'rlut': folderCCSM+'rlut_Amon_CCSM4_G1_r1i1p1_185001-192412.nc',
                      'rlutcs': folderCCSM+'rlutcs_Amon_CCSM4_G1_r1i1p1_185001-192412.nc',
                      }
fluxPaths_G1['ce'] = {'rlut': folderCESM+'rlut_Amon_CESM1-CAM5.1-FV2_G1_r1i1p1_000101-006012.nc', 
                      'rlutcs': folder+'rlutcs_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc', 
                      }                               
fluxPaths_G1['cs'] = {'rlut': folder+'rlut_Amon_CSIRO-Mk3L-1-2_G1_r1i1p1_000101-007012.nc', 
                      'rlutcs': folder+'rlutcs_Amon_CSIRO-Mk3L-1-2_G1_r1i1p1_000101-007012.nc',
                      }    
fluxPaths_G1['gi'] = {'rlut': folder+'rlut_Amon_GISS-E2-R_G1_r1i1p1_185001-191912.nc',   
                      'rlutcs': folder+'rlutcs_Amon_GISS-E2-R_G1_r1i1p1_185001-191912_fixed.nc',
                      }                    
fluxPaths_G1['hg'] = {'rlut': folder+'rlut_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc',   
                      'rlutcs': folder+'rlutcs_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc',
                      }                      
fluxPaths_G1['ip'] = {'rlut': folder+'rlut_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc',   
                      'rlutcs': folder+'rlutcs_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc',
                      }                      
fluxPaths_G1['mi'] = {'rlut': folder+'rlut_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc',   
                      'rlutcs': folder+'rlutcs_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc',
                      }
fluxPaths_G1['mp'] = {'rlut': folder+'rlut_Amon_MPI-ESM-LR_G1_r2i1p1_185001-189912.nc',   
                      'rlutcs': folder+'rlutcs_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc',
                      }                      
fluxPaths_G1['no'] = {'rlut': folderNor+'rlut_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc',   
                      'rlutcs': folder+'rlutcs_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc',
                      }                      
                      
                      
fluxPaths_A4 = OrderedDict()
fluxPaths_A4['bn'] = {'rlut': folder+'rlut_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      'rlutcs': folder+'rlutcs_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      }
fluxPaths_A4['ca'] = {'rlut': folder+'rlut_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      'rlutcs': folder+'rlutcs_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      }
fluxPaths_A4['cc'] = {'rlut': folderCCSM+'rlut_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-200012.nc',
                      'rlutcs': folderCCSM+'rlutcs_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-200012.nc',
                      }
fluxPaths_A4['ce'] = {'rlut': folderCESM+'rlut_Amon_CESM1-CAM5.1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc', 
                      'rlutcs': folder+'rlutcs_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc', 
                      }                  
fluxPaths_A4['cs'] = {'rlut': folder+'rlut_Amon_CSIRO-Mk3L-1-2_abrupt4xCO2_r1i1p1_000101-015012.nc', 
                      'rlutcs': folder+'rlutcs_Amon_CSIRO-Mk3L-1-2_abrupt4xCO2_r1i1p1_000101-015012.nc',
                      }    
fluxPaths_A4['gi'] = {'rlut': folder+'rlut_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912.nc',   
                      'rlutcs': folder+'rlutcs_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912_fixed.nc',
                      }   
fluxPaths_A4['hg'] = {'rlut': folder+'rlut_Amon_HadGEM2-ES_abrupt4xCO2_r1i1p1_185912-191011.nc',   
                      'rlutcs': folder+'rlutcs_Amon_HadGEM2-ES_abrupt4xCO2_r1i1p1_185912-191011.nc',
                      }     
fluxPaths_A4['ip'] = {'rlut': folder+'rlut_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-204912.nc',  
                      'rlutcs': folder+'rlutcs_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-204912.nc',
                      }      
fluxPaths_A4['mi'] = {'rlut': folder+'rlut_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-015012.nc', 
                      'rlutcs': folder+'rlutcs_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-015012.nc',
                      }
fluxPaths_A4['mp'] = {'rlut': folder+'rlut_Amon_MPI-ESM-LR_abrupt4xCO2_r1i1p1_185001-199912.nc',   
                      'rlutcs': folder+'rlutcs_Amon_MPI-ESM-LR_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      }                                            
fluxPaths_A4['no'] = {'rlut': folderNor+'rlut_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-015012.nc',   
                      'rlutcs': folder+'rlutcs_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-015012.nc',
                      }                                  
    

#Create NetCDF Dataset Objects in nested loops, as in "runEBM.py"

#Create NetCDF4 Dataset objects in nested dictionaries using nested loops
#("key" refers to model, "innerKey" refers to variable)          
Datasets_PI = OrderedDict()
for key in fluxPaths_PI.keys():
    Datasets_PI[key] = dict()
    for innerKey in fluxPaths_PI[key].keys():
        Datasets_PI[key][innerKey] = nc4.Dataset(fluxPaths_PI[key][innerKey])   
        
Datasets_G1 = OrderedDict()
for key in fluxPaths_G1.keys():
    Datasets_G1[key] = dict()
    for innerKey in fluxPaths_G1[key].keys():
        Datasets_G1[key][innerKey] = nc4.Dataset(fluxPaths_G1[key][innerKey])     
        
Datasets_A4 = OrderedDict()
for key in fluxPaths_A4.keys():
    Datasets_A4[key] = dict()
    for innerKey in fluxPaths_A4[key].keys():
        Datasets_A4[key][innerKey] = nc4.Dataset(fluxPaths_A4[key][innerKey])           

#Load rlutcs corrections saved by "find_rlut_correction.py"
correction_PI = np.load('npyFiles/rlut_correction_PI.npy').item()
correction_G1 = np.load('npyFiles/rlut_correction_G1.npy').item()
correction_A4 = np.load('npyFiles/rlut_correction_A4.npy').item()


#Calculate LW CRE and save to npy files 
LWCRE_PI = OrderedDict()
LWCRE_G1 = OrderedDict()
LWCRE_A4 = OrderedDict()

models = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']
modelsCorrect_rlutcs = ['bn', 'cc', 'ce', 'no']

loadFromFile = {'bn': False,
                'ca': False,
                'cc': False,
                'ce': False,
                'cs': False,
                'gi': False,
                'hg': False,
                'ip': False, 
                'mi': False,
                'mp': False,
                'no': False
                }

for key in models:
    if not loadFromFile[key]:
        print('Calculating LWCRE: ' + key)
        temp = Datasets_PI[key]['rlut'].variables['rlut'][0:39*12+12,:,:]  #time, lat, lon
        temp_cs = Datasets_PI[key]['rlutcs'].variables['rlutcs'][0:39*12+12,:,:] 
        if key in modelsCorrect_rlutcs:
            temp_cs = temp_cs + correction_PI[key] #Apply same correction to rlutcs that was done in nco for rlut
        if key == 'gi': 
            temp_cs = temp_cs * -1    #rlutcs for GISS had wrong sign
        temp3 = temp_cs - temp #LW up in clear-sky minus all-sky = LW down in all-sky minus clear-sky = LWCRE
        LWCRE_PI[key] = np.mean(temp3, axis=0) #Take time mean 
        LWCRE_PI[key].dump('npyFiles/LWCRE_PI_'+key+'.npy') #dump instead of np.save due to masked arrays
    
        temp = Datasets_G1[key]['rlut'].variables['rlut'][10*12:49*12+12,:,:]  #time, lat, lon
        temp_cs = Datasets_G1[key]['rlutcs'].variables['rlutcs'][10*12:49*12+12,:,:] 
        if key in modelsCorrect_rlutcs:
            temp_cs = temp_cs + correction_G1[key]
        if key == 'gi': 
            temp_cs = temp_cs * -1    
        temp3 = temp_cs - temp 
        LWCRE_G1[key] = np.mean(temp3, axis=0) 
        LWCRE_G1[key].dump('npyFiles/LWCRE_G1_'+key+'.npy')
            
        temp = Datasets_A4[key]['rlut'].variables['rlut'][10*12:49*12+12,:,:]  #time, lat, lon
        temp_cs = Datasets_A4[key]['rlutcs'].variables['rlutcs'][10*12:49*12+12,:,:] 
        if key in modelsCorrect_rlutcs:
            temp_cs = temp_cs + correction_A4[key]
        if key == 'gi': 
            temp_cs = temp_cs * -1    
        temp3 = temp_cs - temp 
        LWCRE_A4[key] = np.mean(temp3, axis=0) 
        LWCRE_A4[key].dump('npyFiles/LWCRE_A4_'+key+'.npy')
    else:
        print('Loading LWCRE: ' + key)
        LWCRE_PI[key] = np.load('npyFiles/LWCRE_PI_'+key+'.npy')
        LWCRE_G1[key] = np.load('npyFiles/LWCRE_G1_'+key+'.npy')
        LWCRE_A4[key] = np.load('npyFiles/LWCRE_A4_'+key+'.npy')
        


    
#Correct for the cloud masking of various longwave feedbacks, calculated using radiative kernels

models_corrected_CRE = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']

#Load model lats and lons (this version for different grid for HadGEM2-ES)
modelLats = np.load('npyFiles/modelLats_ta.npy').item()
modelLons = np.load('npyFiles/modelLons_ta.npy').item()

#Load the corrections for cloud masking (saved by "analyzeKernelResults.py")
dOLR_cloudMasking_G1minusPI = OrderedDict()
dOLR_cloudMasking_A4minusPI = OrderedDict()
for key in models_corrected_CRE:
    print('Loading cloud masking for model: ' + gmf.modelNames[key])
    dOLR_cloudMasking_G1minusPI[key] = np.load('npyFiles/shellKernels_dLW_cloudMasking_G1minusPI_annualMean_'+key+'.npy')
    dOLR_cloudMasking_A4minusPI[key] = np.load('npyFiles/shellKernels_dLW_cloudMasking_A4minusPI_annualMean_'+key+'.npy')
    
#Calculate the corrected change in LW CRE--call it LW cloud adjustment
#LWCRE is defined as positive for net downward increase in LW radiation at TOA--
#The cloud masking was a change in OLR, so I have to ADD this.     
cloudLW_Adjustment_G1minusPI = OrderedDict()
cloudLW_Adjustment_A4minusPI = OrderedDict()
for key in models_corrected_CRE: 
    print('Calculating LW cloud adjustment for model: ' + gmf.modelNames[key])
    if key == 'hg': #Interpolate the HadGEM LW CRE temperatures to midpoint latitudes 
    #(kernel grid is halfway between cloud grid points, so can just take averages of adjacent points)        
        tempPI = (LWCRE_PI[key][1:,:]+LWCRE_PI[key][:-1,:])/2.
        tempG1 = (LWCRE_G1[key][1:,:]+LWCRE_G1[key][:-1,:])/2.
        tempA4 = (LWCRE_A4[key][1:,:]+LWCRE_A4[key][:-1,:])/2.
        cloudLW_Adjustment_G1minusPI[key] = tempG1 - tempPI + dOLR_cloudMasking_G1minusPI[key]
        cloudLW_Adjustment_A4minusPI[key] = tempA4 - tempPI + dOLR_cloudMasking_A4minusPI[key]
    else:
        cloudLW_Adjustment_G1minusPI[key] = LWCRE_G1[key] - LWCRE_PI[key] + dOLR_cloudMasking_G1minusPI[key]
        cloudLW_Adjustment_A4minusPI[key] = LWCRE_A4[key] - LWCRE_PI[key] + dOLR_cloudMasking_A4minusPI[key]
    

#####   MAPS FOR INDIVIDUAL MODELS   #####

#PNG plots for paper supplement
def mapCloudAdjustmentPNG(figNo, key, run1, run2, clevs='default',  save=False):
    w,h = plt.figaspect(1./2.5)  
    plt.figure(figNo, figsize=(w,h))       
    lat = modelLats[key]
    lon = modelLons[key]
    if(run1 == 'G1' and run2 == 'PI'):
        plotData = cloudLW_Adjustment_G1minusPI[key]
    elif(run1 == 'A4' and run2 == 'PI'):
        plotData = cloudLW_Adjustment_A4minusPI[key]
    else:
        print('Haven\'t programmed this combination of runs yet; exiting plot difference function')
        return
    titleString = gmf.modelNames[key]
    #Duplicate edge data to eliminate white strip on left side
    lon = np.append(lon,360.) 
    plotData = np.append(plotData, plotData[:,0:1], axis=1)  
    #Make map
    Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
    Map.drawcoastlines(linewidth=1)
    Map.drawparallels(np.linspace(-90.,90.,7))
    Map.drawmeridians(np.linspace(-150., 150., 11))
    lons, lats = np.meshgrid(lon, lat)
    if clevs == 'default':
        thePlot = Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu_r)
    else: 
        thePlot = Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu_r, levels=clevs, extend='both')
    cbar = plt.colorbar(thePlot)
    cbar.ax.set_ylabel('W m$^{-2}$', fontsize=24)
    tickLocs = np.linspace(min(clevs), max(clevs), len(clevs)-1)
    cbar.set_ticks(tickLocs)
    cbar.set_ticklabels(np.around(tickLocs, 3))
    cbar.ax.tick_params(labelsize=22)
    plt.title(titleString, fontsize=26)
    plt.tight_layout()
    if(save):   
        plt.savefig('plots/individualModels/LW_cloudAdjustment_map_'+run1+'-'+run2+'_'+key+'.png') 


mapCloudAdjustmentPNG(710, 'bn', 'G1', 'PI', clevs=np.linspace(-10, 10, 12), save=True)
mapCloudAdjustmentPNG(810, 'ca', 'G1', 'PI', clevs=np.linspace(-10, 10, 12), save=True)
mapCloudAdjustmentPNG(811, 'cc', 'G1', 'PI', clevs=np.linspace(-10, 10, 12), save=True)
mapCloudAdjustmentPNG(812, 'ce', 'G1', 'PI', clevs=np.linspace(-10, 10, 12), save=True)
mapCloudAdjustmentPNG(813, 'cs', 'G1', 'PI', clevs=np.linspace(-10, 10, 12), save=True)
mapCloudAdjustmentPNG(814, 'gi', 'G1', 'PI', clevs=np.linspace(-10, 10, 12), save=True)
mapCloudAdjustmentPNG(815, 'hg', 'G1', 'PI', clevs=np.linspace(-10, 10, 12), save=True)
mapCloudAdjustmentPNG(816, 'ip', 'G1', 'PI', clevs=np.linspace(-10, 10, 12), save=True)
mapCloudAdjustmentPNG(817, 'mi', 'G1', 'PI', clevs=np.linspace(-10, 10, 12), save=True)
mapCloudAdjustmentPNG(818, 'mp', 'G1', 'PI', clevs=np.linspace(-10, 10, 12), save=True)
mapCloudAdjustmentPNG(819, 'no', 'G1', 'PI', clevs=np.linspace(-10, 10, 12), save=True)


#####   MULTI-MODEL MEAN MAPS   #####

skipMMM_corrected = False
if not skipMMM_corrected:
    # Multi-Model Means
    
    modelsMMM_corrected = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no'] 
    
    #Now take multi-model means.
    #Use "gmf.collocateModels" function.
    #See "multiModelMeanAPRP" for existing example. 
    
    #Lats and lons to interpolate to (1-degree grid)
    interpLats = np.linspace(-86., 86., 173) #Avoiding polar values (which would be masked) to be safe
    interpLons = np.linspace(0., 360., 361)
    
    #Sequence of 3 calls to functions: 
    #The first regrids them to a common grid, 
    #The 2nd takes multi-model mean, 
    #And the 3rd calculates where all but 2 models agree on the sign for stippling purposes.
    
    #Doing all the multi-model mean calculations for G1 minus piControl
    print('Regridding LW cloud adjustment, G1 - piControl: ')
    collocatedAdjustment_G1minusPI = gmf.collocateModels(cloudLW_Adjustment_G1minusPI, modelLats, modelLons, 
                                                              interpLats, interpLons, method='linear')
                                                              #interpLats, interpLons, method='nearest')
    mmmAdjustment_G1minusPI = np.mean(collocatedAdjustment_G1minusPI, axis=2)
    agreeBut2Adjustment_G1minusPI = gmf.arraySignAgreement3D(collocatedAdjustment_G1minusPI, 2)
    
    #Doing all the multi-model mean calculations for A4 minus piControl
    print('Regridding LW cloud adjustment, A4 - piControl: ')
    collocatedAdjustment_A4minusPI = gmf.collocateModels(cloudLW_Adjustment_A4minusPI, modelLats, modelLons, 
                                                              interpLats, interpLons, method='linear')
                                                              #interpLats, interpLons, method='nearest')
    mmmAdjustment_A4minusPI = np.mean(collocatedAdjustment_A4minusPI, axis=2)
    agreeBut2Adjustment_A4minusPI = gmf.arraySignAgreement3D(collocatedAdjustment_A4minusPI, 2)
    
    #Plots
    
    w,h = plt.figaspect(1./2.5)  
    
    #Hatched where less than all but 2 of models agree on sign of change 
    
    #"Big text" versions for paper
    
    #G1 - piControl
    plt.figure(140, figsize=(w,h))
    Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
    Map.drawcoastlines(linewidth=1)
    Map.drawparallels(np.linspace(-90.,90.,7))
    Map.drawmeridians(np.linspace(-150., 150., 11))
    lon = interpLons
    lat = interpLats    
    lons, lats = np.meshgrid(lon, lat)
    plotData = mmmAdjustment_G1minusPI
    cmap = plt.cm.RdBu                         
    clevs=np.linspace(-5., 5., 12)                
    Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu_r, levels=clevs, extend='both') 
    cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
    cbar.ax.set_ylabel('W m$^{-2}$', fontsize=24)
    cbar.ax.tick_params(labelsize=22)   
    plt.title('LW Cloud Adjustment', fontsize=26)   
    #Add hatching (next 2 lines)
    zm = np.ma.masked_where(agreeBut2Adjustment_G1minusPI == True, plotData)
    Map.pcolor(lons, lats, zm, latlon=True, cmap=plt.cm.RdBu_r, alpha=0, hatch='xx') 
    plt.tight_layout()
    plt.savefig('plots/multiModelMean_LW_CloudAdjustment_G1minusPI_bigText.png')
    
    #abrupt4xCO2 - piControl
    plt.figure(150, figsize=(w,h))
    Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
    Map.drawcoastlines(linewidth=1)
    Map.drawparallels(np.linspace(-90.,90.,7))
    Map.drawmeridians(np.linspace(-150., 150., 11))
    lon = interpLons
    lat = interpLats    
    lons, lats = np.meshgrid(lon, lat)
    plotData = mmmAdjustment_A4minusPI
    cmap = plt.cm.RdBu  
    clevs=np.linspace(-10., 10., 12)
    Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu_r, levels=clevs, extend='both')
    cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
    cbar.ax.set_ylabel('W m$^{-2}$', fontsize=24)
    cbar.ax.tick_params(labelsize=22)   
    plt.title('LW Cloud Adjusmtent', fontsize=26)   
    #Add hatching (next 2 lines)
    zm = np.ma.masked_where(agreeBut2Adjustment_A4minusPI == True, plotData)
    Map.pcolor(lons, lats, zm, latlon=True, cmap=plt.cm.RdBu_r, alpha=0, hatch='xx') 
    plt.tight_layout()
    plt.savefig('plots/multiModelMean_LW_CloudAdjustment_A4minusPI_bigText.png')


#####   GLOBAL MEAN CALCULATIONS   #####

skipGlobalMeansCorrected=False
if not skipGlobalMeansCorrected:
        
    def calcGlobalMeanAdjustment(dictName, key):
        Data = dictName[key]
        Data = np.ma.masked_where(np.isnan(Data), Data) #Default outside "convex hull" for regridding was nan--convert this to masked array
                                                        #This statement may make the "hasattr" check below unnecesary. 
        lat = modelLats[key]
        areaWeights = np.cos(lat*np.pi/180)
        areaWeights2D = np.tile(areaWeights,(np.shape(Data)[1],1))
        #print(np.shape(areaWeights2D)) #debug
        areaWeights2D = np.swapaxes(areaWeights2D,0,1)
        if hasattr(Data, 'mask'): #Need to give the area weights the same mask as the data to avoid bias when dividing by sum of weights
            areaWeights2D_mask = np.ma.masked_where(Data.mask == True, areaWeights2D)
            weighted2DMatrix = Data*areaWeights2D_mask
            sumWeighted = np.sum(weighted2DMatrix)
            sumWeights = np.sum(areaWeights2D_mask)
        else:  #don't mask the weights (data not masked array--attempting to access mask will cause error)
            weighted2DMatrix = Data*areaWeights2D
            sumWeighted = np.sum(weighted2DMatrix)
            sumWeights = np.sum(areaWeights2D)
        weightedMean = sumWeighted/sumWeights
        return weightedMean
        
    globalMean_cloudLW_Adjustment_G1minusPI = OrderedDict()
    globalMean_cloudLW_Adjustment_A4minusPI = OrderedDict()
    
    for key in models_corrected_CRE:
        print('Taking global mean LW cloud adjustment (corrected for cloud masking): '+ key)
        globalMean_cloudLW_Adjustment_G1minusPI[key] = calcGlobalMeanAdjustment(cloudLW_Adjustment_G1minusPI, key)
        globalMean_cloudLW_Adjustment_A4minusPI[key] = calcGlobalMeanAdjustment(cloudLW_Adjustment_A4minusPI, key)
        
    #Save to npy files
    np.save('npyFiles/globalMean_cloudLW_Adjustment_G1minusPI.npy', globalMean_cloudLW_Adjustment_G1minusPI)
    np.save('npyFiles/globalMean_cloudLW_Adjustment_A4minusPI.npy', globalMean_cloudLW_Adjustment_A4minusPI)
    
    #Console output
    
    print('Global mean changes in -OLR due to clouds, adjusted for cloud masking')
    
    print('G1 minus piControl (W/m^2): ')
    for key in models_corrected_CRE:
        print(key + ': ' + str(globalMean_cloudLW_Adjustment_G1minusPI[key]))
    
    print('abrupt4xCO2 minus piControl (W/m^2): ')
    for key in models_corrected_CRE:
        print(key + ': ' + str(globalMean_cloudLW_Adjustment_A4minusPI[key]))
        
        
    #Output copied here: (Results listed in Table 3 of paper)
    #G1 minus piControl (W/m^2):  
    #bn: -0.0766409705927
    #ca: -0.26355817659
    #cc: 0.129266158313
    #ce: 0.295863811135
    #cs: -0.244991565257
    #gi: -0.0693880456474
    #hg: -0.149072014191
    #ip: -0.855072644727
    #mi: -0.579599562053
    #mp: -0.54325583803
    #no: -0.0982464632418
    #abrupt4xCO2 minus piControl (W/m^2): 
    #bn: 1.38661131183
    #ca: 3.02381502978
    #cc: -1.05766399014
    #ce: -3.52416148107
    #cs: 0.690112872338
    #gi: 2.02991680236
    #hg: 1.72717359957
    #ip: 0.0895056250141
    #mi: 2.02481857073
    #mp: 1.60924308245
    #no: 0.375397202733
    