# Script to plot multi-model means of APRP results. 
# Will load the npy files created by "scriptUsingAPRPonGeoMIP.py". 

import matplotlib.pyplot as plt
import geomipFunctions as gmf
import numpy as np
from mpl_toolkits.basemap import Basemap
from collections import OrderedDict

#modelKeys = ['bn', 'ca', 'cc', 'ce', 'hg', 'ip', 'mi', 'mp', 'no'] #Include these 9 models in analysis
#Dec. 4, 2017: Exclude BNU until I have replaced r1i1p1 with r3i1p1; otherwise will skew statistics, for "geo" case.
#modelKeys = ['ca', 'cc', 'ce', 'hg', 'ip', 'mi', 'mp', 'no']
#Feb. 18th, 2018: putting BNU back in after doing calculations for r3i1p1
modelKeys = ['bn', 'ca', 'cc', 'ce', 'hg', 'ip', 'mi', 'mp', 'no']
cases = ['geo', 'warm', 'solar']

# Load the 3 APRP output dicts, for "geo" (G1 - piControl), "warm" (abrupt4xCO2 - piControl), 
# and "solar" (abrupt4xCO2 - G1) cases from the npy files
aprp = dict() #Nested dictionaries, one for each case 
aprp['geo'] = OrderedDict()
aprp['warm'] = OrderedDict()
aprp['solar'] = OrderedDict()
for key in modelKeys:
    print('Loading APRP results for model: ' + gmf.modelNames[key])
    aprp['geo'][key] =  np.load('../nobackup/npyFiles/aprp_geo_'+key+'.npy').item()
    aprp['warm'][key] =  np.load('../nobackup/npyFiles/aprp_warm_'+key+'.npy').item()
    aprp['solar'][key] =  np.load('../nobackup/npyFiles/aprp_solar_'+key+'.npy').item()

# Calculate annual means of APRP results, and arrange into dicts for use by multi-model mean functions
# Do this for Cloud, Noncloud and Surface radiative responses for all 3 models
aprpCloud = dict()
for case in cases: 
    aprpCloud[case] = OrderedDict()
    for key in modelKeys:
        aprpCloud[case][key] = np.mean(aprp[case][key]['APRP']['cloud'], axis=0)

aprpNoncloud = dict()
for case in cases: 
    aprpNoncloud[case] = OrderedDict()
    for key in modelKeys:
        aprpNoncloud[case][key] = np.mean(aprp[case][key]['APRP']['noncloud'], axis=0)        
        
aprpSurface = dict()
for case in cases: 
    aprpSurface[case] = OrderedDict()
    for key in modelKeys:
        aprpSurface[case][key] = np.mean(aprp[case][key]['APRP']['surface'], axis=0)        
        
# Obtain dictionaries of the model latitudes and longitudes
modelLats = dict()
modelLons = dict()
for key in modelKeys:
    modelLats[key] = aprp['geo'][key]['Time1_preliminaries']['lat']
    modelLons[key] = aprp['geo'][key]['Time1_preliminaries']['lon']
    #print(modelLons[key]) #Debug #They are from 0 to 360, not -180 to 180.
    
# Generate a 3D array of regridded model data (for each variable and case), then calculate multi-model mean 
# and whether at least 7 models agree on sign
# Will do 2-degree grid for this. 
#interpLats = np.linspace(-90.,90.,91)      #2-degree resolution (for testing--faster)
#interpLons = np.linspace(0., 360., 181) 
interpLats = np.linspace(-90.,90.,181)       #1-degree resolution (for final run)
interpLons = np.linspace(0., 360., 361) 
    
    
collocatedCloud = dict()
mmmCloud = dict()
agree7Cloud = dict()
for case in cases:
    print('Regridding APRP Cloud radiative response for '  + case + ' case:')
    collocatedCloud[case] = gmf.collocateModels(aprpCloud[case], modelLats, modelLons, interpLats, interpLons, method='linear')#method='nearest')
    mmmCloud[case] = np.mean(collocatedCloud[case], axis=2)
    agree7Cloud[case] = gmf.arraySignAgreement3D(collocatedCloud[case], 2)

collocatedNoncloud = dict()
mmmNoncloud = dict()
agree7Noncloud = dict()
for case in cases:
    print('Regridding APRP Noncloud radiative response for '  + case + ' case:')
    collocatedNoncloud[case] = gmf.collocateModels(aprpNoncloud[case], modelLats, modelLons, interpLats, interpLons)
    mmmNoncloud[case] = np.mean(collocatedNoncloud[case], axis=2)
    agree7Noncloud[case] = gmf.arraySignAgreement3D(collocatedNoncloud[case], 2)
    
collocatedSurface = dict()
mmmSurface = dict()
agree7Surface = dict()
for case in cases:
    print('Regridding APRP Surface radiative response for '  + case + ' case:')
    collocatedSurface[case] = gmf.collocateModels(aprpSurface[case], modelLats, modelLons, interpLats, interpLons)
    mmmSurface[case] = np.mean(collocatedSurface[case], axis=2)
    agree7Surface[case] = gmf.arraySignAgreement3D(collocatedSurface[case], 2)




#####   PLOTS   #####

runStrings={'geo': 'G1 - piControl', 'warm': 'abrupt4xCO2 - piControl', 'solar': 'abrupt4xCO2 - G1'}
w,h = plt.figaspect(1./2.5)  

#Plot cloud, noncloud and surface SW radiative response in each of 3 cases, in multi-model mean. 
#Hatched where less than 7 of 9 models agree on sign. (6 of 8 for 12-4-17 run)
#Having experimented with different hatches. and consulted officemate Lauren, used 'xx' version.
#Idea for this hatching method from:
#http://stackoverflow.com/questions/41664850/hatch-area-using-pcolormesh-in-basemap)  

#geo-cloud
plt.figure(0, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
plotData = mmmCloud['geo'] 
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-10., 10., 12)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('W m$^{-2}$')
plt.title('SW cloud radiative response: ' + runStrings['geo'] + ':\nMulti-Model Mean')   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agree7Cloud['geo'] == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
plt.savefig('plots/APRP_mmm_cloud_geo_hatch_xx.png')

#geo-noncloud
plt.figure(1, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
plotData = mmmNoncloud['geo'] 
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-2., 2., 12)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('W m$^{-2}$')
plt.title('SW non-cloud atmosphere radiative response: ' + runStrings['geo'] + ':\nMulti-Model Mean')   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agree7Noncloud['geo'] == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
plt.savefig('plots/APRP_mmm_noncloud_geo_hatch_xx.png')

#geo-surface
plt.figure(2, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
plotData = mmmSurface['geo'] 
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-10., 10., 12)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('W m$^{-2}$')
plt.title('SW surface radiative response: ' + runStrings['geo'] + ':\nMulti-Model Mean')   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agree7Surface['geo'] == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
plt.savefig('plots/APRP_mmm_surface_geo_hatch_xx.png')


#Bigger text versions of the cloud_geo, noncloud_geo and surface_geo maps for AGU 2017 poster and ACP paper
#For ACP paper: standardize from -5 to 5 W/m^2. 
plt.figure(100, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
plotData = mmmCloud['geo'] 
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-5., 5., 12)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('W m$^{-2}$', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(a) SW radiative effect of cloud changes', fontsize=26)   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agree7Cloud['geo'] == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
plt.savefig('plots/APRP_mmm_cloud_geo_hatch_xx_bigText.png')

#geo-noncloud
plt.figure(101, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
plotData = mmmNoncloud['geo'] 
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-5., 5., 12)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('W m$^{-2}$', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(b) SW radiative effect of non-cloud atmosphere changes', fontsize=26)   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agree7Noncloud['geo'] == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
plt.savefig('plots/APRP_mmm_noncloud_geo_hatch_xx_bigText.png')

#geo-surface
plt.figure(102, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
plotData = mmmSurface['geo'] 
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-5., 5., 12)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('W m$^{-2}$', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(c) SW radiative effect of surface albedo changes', fontsize=26)   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agree7Surface['geo'] == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
plt.savefig('plots/APRP_mmm_surface_geo_hatch_xx_bigText.png')


#warm-cloud
plt.figure(3, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
plotData = mmmCloud['warm'] 
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-20., 20., 12)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('W m$^{-2}$')
plt.title('SW cloud radiative response: ' + runStrings['warm'] + ':\nMulti-Model Mean')   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agree7Cloud['warm'] == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
plt.savefig('plots/APRP_mmm_cloud_warm_hatch_xx.png')

#warm-noncloud
plt.figure(4, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
plotData = mmmNoncloud['warm'] 
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-4., 4., 12)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('W m$^{-2}$')
plt.title('SW non-cloud atmosphere radiative response: ' + runStrings['warm'] + ':\nMulti-Model Mean')   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agree7Noncloud['warm'] == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
plt.savefig('plots/APRP_mmm_noncloud_warm_hatch_xx.png')

#warm-surface
plt.figure(5, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
plotData = mmmSurface['warm'] 
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-20., 20., 12)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('W m$^{-2}$')
plt.title('SW surface radiative response: ' + runStrings['warm'] + ':\nMulti-Model Mean')   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agree7Surface['warm'] == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
plt.savefig('plots/APRP_mmm_surface_warm_hatch_xx.png')


#solar-cloud
plt.figure(6, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
plotData = mmmCloud['solar'] 
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-20., 20., 12)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('W m$^{-2}$')
plt.title('SW cloud radiative response: ' + runStrings['solar'] + ':\nMulti-Model Mean')   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agree7Cloud['solar'] == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
plt.savefig('plots/APRP_mmm_cloud_solar_hatch_xx.png')

#solar-noncloud
plt.figure(7, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
plotData = mmmNoncloud['solar'] 
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-4., 4., 12)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('W m$^{-2}$')
plt.title('SW non-cloud atmosphere radiative response: ' + runStrings['solar'] + ':\nMulti-Model Mean')   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agree7Noncloud['solar'] == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
plt.savefig('plots/APRP_mmm_noncloud_solar_hatch_xx.png')

#solar-surface
plt.figure(8, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
plotData = mmmSurface['solar'] 
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-20., 20., 12)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('W m$^{-2}$')
plt.title('SW surface radiative response: ' + runStrings['solar'] + ':\nMulti-Model Mean')   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agree7Surface['solar'] == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
plt.savefig('plots/APRP_mmm_surface_solar_hatch_xx.png')



plt.show()
