# Script to look at time series of APRP results in the G1 - piControl case, 
# to see if there is any meaningful drift. 
#
# Rick Russotto
# Started March 1, 2017


import APRP as aprp
import netCDF4 as nc4
import os
import matplotlib.pyplot as plt
import geomipFunctions as gmf
import numpy as np
from mpl_toolkits.basemap import Basemap
from collections import OrderedDict #need values & keys in correct order for plot legends


#####   FUNCTION DEFINITIONS   #####

# Function for global mean of a 2D array with lat, lon dimensions (APRP output, already extracted)
# ("lat" is array of latitudes; this is in the APRP output dictionary)
def globalMean(array, lat):
    areaWeights = np.cos(lat*np.pi/180)
    areaWeights2D = np.tile(areaWeights,(np.shape(array)[1],1))
    areaWeights2D = np.swapaxes(areaWeights2D,0,1)
    weighted2DMatrix = array*areaWeights2D
    sumWeighted = np.sum(weighted2DMatrix)
    sumWeights = np.sum(areaWeights2D)
    weightedMean = sumWeighted/sumWeights
    return weightedMean       
    
    

# Wrapper function to run APRP in a loop. 
#
# Input arguments:
# "Paths": dictionary of paths to GCM output (see APRP code)
# "baseline": control run to compare to (piControl, for CO2 forcing, or G1, for solar forcing)
# "perturb": run for which we are examining every year (abrupt4xCO2)
# Year indexing starts from zero. 
#
# Returns: (not full APRP output array--would be too much data)
# A dictionary with: cloudLocal, noncloudLocal, surfaceLocal (all 3D arrays with named APRP output variable for each perturbation year), 
#                    and 1D arrays containing global means of the above in each year. 
def yearlyAPRP(baselinePaths, baselineFirstYear, baselineLastYear, perturbPaths, perturbFirstYear, perturbLastYear):
    numYearsPerturb = perturbLastYear - perturbFirstYear + 1
    
    #Convert years to months for the baseline
    baselineFirstMonth = baselineFirstYear*12
    baselineLastMonth = (baselineLastYear+1)*12-1
    
    #Loop over the years in the perturbation run, populating 3D arrays for local reusults and 1D arrays for global mean results
    for i in range(0, numYearsPerturb):
        
        #Print status (so I can see how long it's taking)
        print('Calculating APRP for year ' + str(i+1) + '/' + str(numYearsPerturb) + ' of perturbation range')        
        
        #Find month indices for the current year
        perturbFirstMonth = perturbFirstYear*12 + i*12
        #perturbLastMonth = (perturbLastYear+1)*12 + i*12 - 1
        perturbLastMonth = (perturbFirstYear+1)*12 + i*12 - 1 #fix May 8th, 2018
        
        #Run APRP
        aprpDict = aprp.aprp_main(baselinePaths, baselineFirstMonth, baselineLastMonth, perturbPaths, perturbFirstMonth, perturbLastMonth)
        
        #Extract latitude and longitude
        lat = aprpDict['Time1_preliminaries']['lat']
        numLats = len(lat)
        lon = aprpDict['Time1_preliminaries']['lon']
        numLons = len(lon)
        
        #Initialize output arrays, if not done yet
        if i == 0:
            cloudLocal = np.zeros((numLats, numLons, numYearsPerturb))
            noncloudLocal = np.zeros((numLats, numLons, numYearsPerturb))
            surfaceLocal = np.zeros((numLats, numLons, numYearsPerturb))
            cloudGlobal = np.zeros(numYearsPerturb)
            noncloudGlobal = np.zeros(numYearsPerturb)
            surfaceGlobal = np.zeros(numYearsPerturb)
        
        #Populate the output arrays for this year (taking annual mean, and global mean if appropriate)
        cloudLocal[:,:,i] = np.mean(aprpDict['APRP']['cloud'], axis=0)
        noncloudLocal[:,:,i] = np.mean(aprpDict['APRP']['noncloud'], axis=0)
        surfaceLocal[:,:,i] = np.mean(aprpDict['APRP']['surface'], axis=0)
        cloudGlobal[i] = globalMean(cloudLocal[:,:,i], lat)
        noncloudGlobal[i] = globalMean(noncloudLocal[:,:,i], lat)
        surfaceGlobal[i] = globalMean(surfaceLocal[:,:,i], lat)
        
    #Now out of the loop over the years, assemble output arrays into a dict and return. 
    outputDict = dict()
    outputDict['cloudLocal'] = cloudLocal
    outputDict['noncloudLocal'] = noncloudLocal
    outputDict['surfaceLocal'] = surfaceLocal
    outputDict['cloudGlobal'] = cloudGlobal
    outputDict['noncloudGlobal'] = noncloudGlobal
    outputDict['surfaceGlobal'] = surfaceGlobal
    
    return outputDict
    
#####   SPECIFY GCM OUTPUT LOCATIONS   #####

folder = '../nobackup/'

#G1
fluxPaths_G1 = OrderedDict()
#fluxPaths_G1['bn'] = {'clt': folder+'clt_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc', 
#                      'rsds': folder+'rsds_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc',
#                      'rsdscs': folder+'rsdscs_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc',
#                      'rsdt': folder+'rsdt_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc',
#                      'rsus': folder+'rsus_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc',
#                      'rsuscs': folder+'rsuscs_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc',
#                      'rsut': folder+'rsut_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc',
#                      'rsutcs': folder+'rsutcs_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc'
#                      }
fluxPaths_G1['bn'] = {'clt': folder+'clt_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc', 
                      'rsds': folder+'rsds_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      'rsdscs': folder+'rsdscs_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      'rsdt': folder+'rsdt_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      'rsus': folder+'rsus_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      'rsuscs': folder+'rsuscs_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      'rsut': folder+'rsut_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      'rsutcs': folder+'rsutcs_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc'
                      }
fluxPaths_G1['ca'] = {'clt': folder+'clt_Amon_CanESM2_G1_r1i1p1_185001-194912.nc', 
                      'rsds': folder+'rsds_Amon_CanESM2_G1_r1i1p1_185001-194912.nc',
                      'rsdscs': folder+'rsdscs_Amon_CanESM2_G1_r1i1p1_185001-194912.nc',
                      'rsdt': folder+'rsdt_Amon_CanESM2_G1_r1i1p1_185001-194912.nc',
                      'rsus': folder+'rsus_Amon_CanESM2_G1_r1i1p1_185001-194912.nc',
                      'rsuscs': folder+'rsuscs_Amon_CanESM2_G1_r1i1p1_185001-194912.nc',
                      'rsut': folder+'rsut_Amon_CanESM2_G1_r1i1p1_185001-194912.nc',
                      'rsutcs': folder+'rsutcs_Amon_CanESM2_G1_r1i1p1_185001-194912.nc'
                      }                 
fluxPaths_G1['cc'] = {'clt': folder+'clt_Amon_CCSM4_G1_r1i1p1_185001-192412.nc', 
                      'rsds': folder+'rsds_Amon_CCSM4_G1_r1i1p1_185001-192412.nc',
                      'rsdscs': folder+'rsdscs_Amon_CCSM4_G1_r1i1p1_185001-192412.nc',
                      'rsdt': folder+'rsdt_Amon_CCSM4_G1_r1i1p1_185001-192412.nc',
                      'rsus': folder+'rsus_Amon_CCSM4_G1_r1i1p1_185001-192412.nc',
                      'rsuscs': folder+'rsuscs_Amon_CCSM4_G1_r1i1p1_185001-192412.nc',
                      'rsut': folder+'rsut_Amon_CCSM4_G1_r1i1p1_185001-192412.nc',
                      'rsutcs': folder+'rsutcs_Amon_CCSM4_G1_r1i1p1_185001-192412.nc'
                      }    
fluxPaths_G1['ce'] = {'clt': folder+'clt_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc', 
                      'rsds': folder+'rsds_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-005912.nc',
                      'rsdscs': folder+'rsdscs_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc',
                      'rsdt': folder+'rsdt_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc',
                      'rsus': folder+'rsus_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-005912.nc',
                      'rsuscs': folder+'rsuscs_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc',
                      'rsut': folder+'rsut_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc',
                      'rsutcs': folder+'rsutcs_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc'
                      }      
fluxPaths_G1['hg'] = {'clt': folder+'clt_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc', 
                      'rsds': folder+'rsds_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc',
                      'rsdscs': folder+'rsdscs_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc',
                      'rsdt': folder+'rsdt_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc',
                      'rsus': folder+'rsus_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc',
                      'rsuscs': folder+'rsuscs_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc',
                      'rsut': folder+'rsut_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc',
                      'rsutcs': folder+'rsutcs_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc'
                      }                          
fluxPaths_G1['ip'] = {'clt': folder+'clt_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc', 
                      'rsds': folder+'rsds_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc',
                      'rsdscs': folder+'rsdscs_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc',
                      'rsdt': folder+'rsdt_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc',
                      'rsus': folder+'rsus_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc',
                      'rsuscs': folder+'rsuscs_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc',
                      'rsut': folder+'rsut_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc',
                      'rsutcs': folder+'rsutcs_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc'
                      }                    
fluxPaths_G1['mi'] = {'clt': folder+'clt_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc', 
                      'rsds': folder+'rsds_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc',
                      'rsdscs': folder+'rsdscs_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc',
                      'rsdt': folder+'rsdt_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc',
                      'rsus': folder+'rsus_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc',
                      'rsuscs': folder+'rsuscs_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc',
                      'rsut': folder+'rsut_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc',
                      'rsutcs': folder+'rsutcs_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc'
                      }        
fluxPaths_G1['mp'] = {'clt': folder+'clt_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc', 
                      'rsds': folder+'rsds_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc',
                      'rsdscs': folder+'rsdscs_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc',
                      'rsdt': folder+'rsdt_Amon_MPI-ESM-LR_G1_r2i1p1_185001-189912.nc',
                      'rsus': folder+'rsus_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc',
                      'rsuscs': folder+'rsuscs_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc',
                      'rsut': folder+'rsut_Amon_MPI-ESM-LR_G1_r2i1p1_185001-189912.nc',
                      'rsutcs': folder+'rsutcs_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc'
                      }                              
fluxPaths_G1['no'] = {'clt': folder+'clt_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc', 
                      'rsds': folder+'rsds_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc',
                      'rsdscs': folder+'rsdscs_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc',
                      'rsdt': folder+'rsdt_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc',
                      'rsus': folder+'rsus_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc',
                      'rsuscs': folder+'rsuscs_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc',
                      'rsut': folder+'rsut_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc',
                      'rsutcs': folder+'rsutcs_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc'
                      }                              
                      
# piControl                      
fluxPaths_PI = OrderedDict()
fluxPaths_PI['bn'] = {'clt': folder+'clt_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc', 
                      'rsds': folder+'rsds_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      'rsdscs': folder+'rsdscs_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      'rsdt': folder+'rsdt_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      'rsus': folder+'rsus_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      'rsuscs': folder+'rsuscs_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      'rsut': folder+'rsut_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      'rsutcs': folder+'rsutcs_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc'
                      }    
fluxPaths_PI['ca'] = {'clt': folder+'clt_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc', 
                      'rsds': folder+'rsds_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc',
                      'rsdscs': folder+'rsdscs_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc',
                      'rsdt': folder+'rsdt_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc',
                      'rsus': folder+'rsus_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc',
                      'rsuscs': folder+'rsuscs_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc',
                      'rsut': folder+'rsut_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc',
                      'rsutcs': folder+'rsutcs_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc'
                      }    
fluxPaths_PI['cc'] = {'clt': folder+'clt_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc', 
                      'rsds': folder+'rsds_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc',
                      'rsdscs': folder+'rsdscs_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc',
                      'rsdt': folder+'rsdt_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc',
                      'rsus': folder+'rsus_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc',
                      'rsuscs': folder+'rsuscs_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc',
                      'rsut': folder+'rsut_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc',
                      'rsutcs': folder+'rsutcs_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc'
                      }    
fluxPaths_PI['ce'] = {'clt': folder+'clt_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc', 
                      'rsds': folder+'rsds_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc',
                      'rsdscs': folder+'rsdscs_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc',
                      'rsdt': folder+'rsdt_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc',
                      'rsus': folder+'rsus_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc',
                      'rsuscs': folder+'rsuscs_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc',
                      'rsut': folder+'rsut_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc',
                      'rsutcs': folder+'rsutcs_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc'
                      }                   
fluxPaths_PI['hg'] = {'clt': folder+'clt_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc', 
                      'rsds': folder+'rsds_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc',
                      'rsdscs': folder+'rsdscs_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc',
                      'rsdt': folder+'rsdt_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc',
                      'rsus': folder+'rsus_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc',
                      'rsuscs': folder+'rsuscs_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc',
                      'rsut': folder+'rsut_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc',
                      'rsutcs': folder+'rsutcs_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc'
                      }                          
fluxPaths_PI['ip'] = {'clt': folder+'clt_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc', 
                      'rsds': folder+'rsds_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc',
                      'rsdscs': folder+'rsdscs_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc',
                      'rsdt': folder+'rsdt_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc',
                      'rsus': folder+'rsus_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc',
                      'rsuscs': folder+'rsuscs_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc',
                      'rsut': folder+'rsut_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc',
                      'rsutcs': folder+'rsutcs_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc'
                      }          
fluxPaths_PI['mi'] = {'clt': folder+'clt_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc', 
                      'rsds': folder+'rsds_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc',
                      'rsdscs': folder+'rsdscs_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc',
                      'rsdt': folder+'rsdt_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc',
                      'rsus': folder+'rsus_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc',
                      'rsuscs': folder+'rsuscs_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc',
                      'rsut': folder+'rsut_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc',
                      'rsutcs': folder+'rsutcs_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc'
                      }        
fluxPaths_PI['mp'] = {'clt': folder+'clt_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-203512.nc', 
                      'rsds': folder+'rsds_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-203512.nc',
                      'rsdscs': folder+'rsdscs_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-203512.nc',
                      'rsdt': folder+'rsdt_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-203512.nc',
                      'rsus': folder+'rsus_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-203512.nc',
                      'rsuscs': folder+'rsuscs_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-203512.nc',
                      'rsut': folder+'rsut_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-203512.nc',
                      'rsutcs': folder+'rsutcs_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-203512.nc'
                      }                            
fluxPaths_PI['no'] = {'clt': folder+'clt_Amon_NorESM1-M_piControl_r1i1p1_070001-115012.nc', 
                      'rsds': folder+'rsds_Amon_NorESM1-M_piControl_r1i1p1_070001-115012.nc',
                      'rsdscs': folder+'rsdscs_Amon_NorESM1-M_piControl_r1i1p1_070001-115012.nc',
                      'rsdt': folder+'rsdt_Amon_NorESM1-M_piControl_r1i1p1_070001-115012.nc',
                      'rsus': folder+'rsus_Amon_NorESM1-M_piControl_r1i1p1_070001-115012.nc',
                      'rsuscs': folder+'rsuscs_Amon_NorESM1-M_piControl_r1i1p1_070001-115012.nc',
                      'rsut': folder+'rsut_Amon_NorESM1-M_piControl_r1i1p1_070001-115012.nc',
                      'rsutcs': folder+'rsutcs_Amon_NorESM1-M_piControl_r1i1p1_070001-115012.nc'
                      }                             
                      

#####   RUN APRP OR LOAD RESULTS   #####

models = ['bn', 'ca', 'cc', 'ce', 'hg', 'ip', 'mi', 'mp', 'no'] #Since this is a list, it is ordered

loadNpyFile_geo = {'bn': True,
                   'ca': True, 
                   'cc': True, 
                   'ce': True,
                   'hg': True,
                   'ip': True,
                   'mi': True,
                   'mp': True,
                   'no': True}

        
#Run "geo" case        
resultsAPRP_geo = OrderedDict()
for key in models:
    if not(loadNpyFile_geo[key]):
        resultsAPRP_geo[key] = yearlyAPRP(fluxPaths_PI[key], 0, 39, fluxPaths_G1[key], 0, 49)
        print('Finished yearly APRP calculation (G1 - piControl): ' + gmf.modelNames[key])
        np.save('../nobackup/npyFiles/aprp_yearly_geo_'+key+'.npy', resultsAPRP_geo[key])
    else:
        resultsAPRP_geo[key] = np.load('../nobackup/npyFiles/aprp_yearly_geo_'+key+'.npy').item()
        print('Loaded APRP results (G1 - piControl): ' + gmf.modelNames[key])        
        
        
        

#####   MAKE AND SAVE PLOTS   #####
        

##Plot time series of the global mean results

for i in range(0,len(models)):
    plt.figure(i)
    l1, = plt.plot(np.arange(1,51), resultsAPRP_geo[models[i]]['cloudGlobal'], 'k', linewidth=2)
    l2, = plt.plot(np.arange(1,51), resultsAPRP_geo[models[i]]['noncloudGlobal'], 'r', linewidth=2)
    l3, = plt.plot(np.arange(1,51), resultsAPRP_geo[models[i]]['surfaceGlobal'], 'b', linewidth=2)
    plt.hlines(0, 0, 50, 'k', linestyles='--')
    plt.title('APRP responses in G1 (global mean): ' + gmf.modelNames[models[i]])
    plt.xlabel('Year')
    plt.ylabel('Radiative Response (W m$^{-2}$)')
    plt.savefig('plots/individualModels/APRP_yearly_geo_globalMean_'+models[i]+'_nolegend.pdf')
    plt.legend([l1, l2, l3], ['cloud', 'non-cloud atmosphere', 'surface albedo'], loc='best')
    plt.savefig('plots/individualModels/APRP_yearly_geo_globalMean_'+models[i]+'.pdf')
    
    
    
#(For paper supplement, the individual model plots were arranged on one page using LaTeX)


plt.show()
