# Script to regrid hybrid sigma height coordinates to height coordinates for the HadGEM2-ES model, and calculate zonal mean. 
#
# Rick Russotto
# Started 9 February 2018

import numpy as np
import netCDF4 as nc4
import matplotlib.pyplot as plt
import geomipFunctions as gmf

#z = a + b*orog

#38 levels
#
#
b = np.array([0.99771648645401, 0.990881502628326, 0.979542553424835, 0.9637770652771, 
              0.943695485591888, 0.919438362121582, 0.891178011894226, 
              0.859118342399597, 0.823493480682373, 0.784570515155792, 
              0.742646217346191, 0.698050200939178, 0.651142716407776, 
              0.602314412593842, 0.55198872089386, 0.500619947910309, 0.44869339466095, 
              0.39672577381134, 0.34526526927948, 0.294891387224197, 0.24621507525444, 
              0.199878215789795, 0.156554222106934, 0.116947874426842, 
              0.0817952379584312, 0.0518637150526047, 0.0279368180781603, 
              0.0107164792716503, 0.00130179093685001, 0, 0, 0, 0, 0, 0, 0, 0, 0 ])

#Don't need surface pressure, just 3D orography, which is saved in the "cl" files. 
#But NetCDF file doesn't say what a is! This would have to be 1 for each level.


# Heights (m) given in Martin et al. (2006) -- these must be the a values...
# (More recent paper for HadGEM2-ES implies this hasn't changed--still 38 levels)

# In the ncdump for the "cl" files, "lev" (which represents a) is in ascending order. 
# I originally had it in descending order but it should be ascending. 
# The numbers are the same as the paper table, within reason.

a = np.flipud(np.array([39254.8,
                        32908.7,
                        29219.1,
                        26583.6,
                        24458.3,
                        22626.1,
                        20990.2,
                        19503.0,
                        18138.6,
                        16875.3,
                        15694.6,
                        14580.8,
                        13520.0,
                        12500.0,
                        11520.0,
                        10580.0,
                        9680.0,
                        8820.0,
                        8000.0,
                        7220.0,
                        6480.0,
                        5780.0,
                        5120.0,
                        4500.0,
                        3920.0,
                        3380.0,
                        2880.0,
                        2420.0,
                        2000.0,
                        1620.0,
                        1280.0,
                        980.0,
                        720.0,
                        500.0,
                        320.0,
                        180.0,
                        80.0,
                        20.0]))
              

              
#Levels to regrid to (avoid duplication with "a" which are standard heights in absence of terrain): 
zLevels = np.flipud(np.array([25, 100, 300, 525, 750, 1000, 1500, 2250, 3000, 4000, 5000, 6000, 7000, 
                              8500, 10000, 14000, 18000, 22000, 26000, 30000, 34000, 38000]))

#Load the cloud output into NetCDF Dataset objects
fpath_cl_PI = '../nobackup/cl_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc' \
fpath_cl_G1 = '../nobackup/cl_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc'
fpath_cl_A4 = '../nobackup/cl_Amon_HadGEM2-ES_abrupt4xCO2_r1i1p1_185912-191011.nc'

print('Loading NetCDF files (cl)') 
Dataset_cl_PI = nc4.Dataset(fpath_cl_PI) 
Dataset_cl_G1 = nc4.Dataset(fpath_cl_G1)
Dataset_cl_A4 = nc4.Dataset(fpath_cl_A4)


#Function to regrid these files to height coordinates based on the orography in the files.
#Based on the "zonalMeanMultiYearPressureVaryingVariable" function from the geomipFunctions module.
#But don't take zonal or time mean yet, so that I can convert to pressure coordinates later. 

# Function to interpolate height-varying model output variables to desired pressure levels.
# Takes some inspiration from code by Dan Vimont at
# http://www.aos.wisc.edu/~dvimont/matlab/
#
# This version is for hybrid sigma height vertical coordinates.
# 
#
# Inputs: 
# Data: NetCDF4 Dataset object containing the cloud outputnp.save(npyFolder+'zonalMeans_cl_G1_'+key+'.npy', zonalMeans_cl_G1[key])
# zLevels: height levels to interpolate to, in m. 
# firstYear: first year of the multi-year mean (measured from start of run)
# lastYear:  last    "  "   "    "     "    "
# A, B: parameters for hybrid sigma coordinates, defined above
#
# Output:
# A 4-D Numpy array with latitude on one axis and pressure on the other, and NaNs where no data exist 
# due to terrain (i.e. over Antarctica). Dimension order: time, height, latitude, longitude
def regridHybridSigmaHeightToHeightCoords(Data, zLevels, firstYear, lastYear, A, B):
            
    # Extract the data from the objects as well as the surface pressure
    datavar = Data.variables['cl'][firstYear*12:lastYear*12+12,:,:,:]   #time, lev, lat, lon
    orog = Data.variables['orog'][:] #lat, lon
    
    # Calculate height at the points where the Data are located.
    # Conversion from hybrid sigma levels to height levels
    # based on equation: #z[i,j,k] = a[k] + b[k]*orog[i,j] 
    # No time variation here, although the data does so will need to use broadcasting at some point. 
    
    BkOij = B[:,None,None]*orog[None,:,:] #Result: 3D array (lev, lat, lon)
    heightMat3D = A[:,None,None] + BkOij
    #Will need to "repmat" the heightMat matrix for every time. 
    heightMat = heightMat3D[None,:,:,:]*np.ones(np.shape(datavar))
    
    #2-9-18: stopped here. 
    
    # Okay, now the hard part:
    # Interpolate the data from the model's native height levels to the desired height levels. 
    # The model's pressure levels vary with latitude and longitude, whereas we need consistent
    # pressure levels for time and zonal averaging. So we have to do a linear interpolation in the 
    # vertical coordinate that varies with latitude and longitude. But doing this in nested loops
    # is way too slow, so we need to use matrix operations. 
    #
    # Further complicating the picture is the fact that sometimes the desired height level lies outside
    # the range of the model height levels, due to terrain. To account for this we need to put
    # nans in these places where there is no data, and use nanmean at the end for zonal and time mean.  
    #
    # General strategy: only one loop, over the new pressure levels. At each desired pressure level, calculate 
    # the difference between the native height and the desired height, and find the vertical 
    # indices of the native height closest to the desired height above and below. Find the native height and 
    # the data values corresponding to these indices in order to do the linear interpolation. In order to put nans 
    # where there is no data, keep track of indices columns where all native height are either 
    # above or below the desired height, and slot in nans in the appropriate place right before the final 
    # interpolation calculation.

    # First: reshape the native height and data into 2D arrays, with time/latitude/longitude all on one axis
    # and vertical coordinate on the other axis. This makes it simpler to extract data at 
    # the particular vertical index we want (which varies with time, latitude, and longitude) later on. 
    # But we will still use the 4D arrays in other parts of the calculation.
    s = np.shape(heightMat)
    numTimes = s[0]
    numLevs = s[1]
    numLats = s[2]
    numLons = s[3]    
    #Make vertical level the last axis so that columns remain intact when matrices are reshaped    
    height2D = np.swapaxes(heightMat,1,3) #Now it's time, lon, lat, level
    height2D = np.reshape(height2D, (numTimes*numLats*numLons, numLevs))
    data2D = np.swapaxes(datavar,1,3)
    data2D = np.reshape(data2D, (numTimes*numLats*numLons, numLevs))
    
    ### --- Everything below this line didn't need to be modified for this model --- ###
        
    # Preallocate array to hold interpolated data
    interpMat = np.empty((heightMat.shape[0], len(zLevels), heightMat.shape[2], heightMat.shape[3]))
    
    # Now: loop over the desired height levels and interpolate data to each one
    for k in range(0, len(zLevels)):
        print('Interpolating to ' + str(zLevels[k]) + 'm')
        
        # This code block: find the upper boundaries of the native height and data for interpolation
        print('Positive side')
        heightMatDiffPos = heightMat - zLevels[k] #Result: 4D array of differences between native and desired height.
                                                      #Positive values: higher native height than the level we're filling
                                                      #Negative values: lower  native height than the level we're filling
        # We're only interested in positive values (higher pressure end) for now, 
        # so set negative ones to a very high number and then find the index associated with the minimum along the 
        # vertical coordinate. 
        heightMatDiffPos[heightMatDiffPos < 0] = 1.e10
        upperIndex = np.argmin(heightMatDiffPos, axis=1)  #upperIndex is 3D array in time, lat, lon
        # Next: Record indices where we're trying to interpolate to greater than the maximum native height in the column.
        # If this is the case, every value of heightMatDiffPos in the column will have been set to 1.e10, so the 
        # difference between the max and min in the column will be zero.
        # We'll create an array of boolean values that are true if this is one such column. They'll be used
        # later to slot in nans.
        nanUpperIndex = np.max(heightMatDiffPos, axis=1) - np.min(heightMatDiffPos,axis=1) #3D array: time, lat, lon
        nanUpperIndexBool = np.ones(np.shape(nanUpperIndex), dtype = bool)
        nanUpperIndexBool[nanUpperIndex >= 1.e-99] = False
        # Now, convert the 3D arrays containing vertical indices of interest and boolean values for nans to 1D vectors
        upperIndex1D = np.swapaxes(upperIndex, 1,2) #switched lat and lon to match reshaped matrices above        
        upperIndex1D = np.reshape(upperIndex1D, numTimes*numLats*numLons) #Convert to a vector
        nanUpperIndex1D = np.swapaxes(nanUpperIndexBool, 1,2)
        nanUpperIndex1D = np.reshape(nanUpperIndex1D, numTimes*numLats*numLons)
        # Now, extract the native height and data values at the upper bound indices we found
        # (I tested this method for extracting data using a sample 2D array in the command line)
        upperHeightBound = height2D[range(0,len(upperIndex1D)),upperIndex1D] #Result: 1D vector        
        upperDataBound = data2D[range(0,len(upperIndex1D)),upperIndex1D]
        # Set the height and data boundary data to nans where we are trying to interpolate to outside the data range
        upperHeightBound[nanUpperIndex1D] = np.nan        
        upperDataBound[nanUpperIndex1D] = np.nan 
        #print(upperDataBound) #debug output
    
        # This code block: same as above but for the lower boundaries. (far less comments)
        print('Negative side')
        heightMatDiffNeg = heightMat - zLevels[k]        
        heightMatDiffNeg[heightMatDiffNeg > 0] = 1.e10 #This time we are only interested in negative values
        lowerIndex = np.argmin(np.abs(heightMatDiffNeg), axis=1) 
        nanLowerIndex = np.max(heightMatDiffNeg, axis=1) - np.min(heightMatDiffNeg,axis=1) 
        nanLowerIndexBool = np.ones(np.shape(nanLowerIndex), dtype = bool)
        nanLowerIndexBool[nanLowerIndex >= 1.e-99] = False
        lowerIndex1D = np.swapaxes(lowerIndex, 1,2) 
        lowerIndex1D = np.reshape(lowerIndex1D, numTimes*numLats*numLons) 
        nanLowerIndex1D = np.swapaxes(nanLowerIndexBool, 1,2)
        nanLowerIndex1D = np.reshape(nanLowerIndex1D, numTimes*numLats*numLons)        
        lowerHeightBound = height2D[range(0,len(lowerIndex1D)),lowerIndex1D]         
        lowerDataBound = data2D[range(0,len(lowerIndex1D)),lowerIndex1D]
        lowerHeightBound[nanLowerIndex1D] = np.nan        
        lowerDataBound[nanLowerIndex1D] = np.nan
        #print(lowerDataBound) #debug output
        
        # Now: linearly interpolate the data in height space (linearly for height; interpolated in log pressure space
        #      in pressure version)
        interpVec = lowerDataBound+(upperDataBound-lowerDataBound)*(zLevels[k]-lowerHeightBound)/(upperHeightBound-lowerHeightBound)
        
        # Finally: Reshape to 3D matrix to put in the later 4D matrix
        interpSlice = np.reshape(interpVec, (numTimes, numLons, numLats)) #time, lon, lat for consistency with above
        interpSlice = np.swapaxes(interpSlice,1,2) # switch lat and lon again
        interpMat[:,k,:,:] = interpSlice #Populate the returned matrix
        
        print('Finished k = ' + str(k))    
    
    #Don't do zonal & time mean yet--interested in intermediate result. 
    
    return interpMat
    
    
#Do regridding and plot results in height coordinates, to see what they look like and make sure it worked, before attempting to convert to pressure coordinates. 
#Save as npy files so I don't have to do regridding twice. 
    
#(4D files for low/middle/high cloud fraction maps, zonal mean plots for the multi-panel cloud fraction change plots)
    
print('Vertical regridding (height coordinates) for HadGEM2-ES for piControl')
cl_4D_heightCoords_hg_PI = regridHybridSigmaHeightToHeightCoords(Dataset_cl_PI, zLevels, 0, 39, a, b)
np.save('../nobackup/npyFiles/cl_4D_heightCoords_hg_PI.npy', cl_4D_heightCoords_hg_PI)

print('Vertical regridding (height coordinates) for HadGEM2-ES for G1')
cl_4D_heightCoords_hg_G1 = regridHybridSigmaHeightToHeightCoords(Dataset_cl_G1, zLevels, 10, 49, a, b)
np.save('../nobackup/npyFiles/cl_4D_heightCoords_hg_G1.npy', cl_4D_heightCoords_hg_G1)

print('Vertical regridding (height coordinates) for HadGEM2-ES for abrupt4xCO2')
cl_4D_heightCoords_hg_A4 = regridHybridSigmaHeightToHeightCoords(Dataset_cl_A4, zLevels, 10, 49, a, b)
np.save('../nobackup/npyFiles/cl_4D_heightCoords_hg_A4.npy', cl_4D_heightCoords_hg_A4)

#Once regridded and saved, load npy files here. 
cl_4D_heightCoords_hg_PI = np.load('../nobackup/npyFiles/cl_4D_heightCoords_hg_PI.npy')
cl_4D_heightCoords_hg_G1 = np.load('../nobackup/npyFiles/cl_4D_heightCoords_hg_G1.npy')
cl_4D_heightCoords_hg_A4 = np.load('../nobackup/npyFiles/cl_4D_heightCoords_hg_A4.npy')

#Now do zonal means and plot

cl_zonalMean_heightCoords_hg_PI = np.nanmean(np.nanmean(cl_4D_heightCoords_hg_PI, axis=3),axis=0)
cl_zonalMean_heightCoords_hg_G1 = np.nanmean(np.nanmean(cl_4D_heightCoords_hg_G1, axis=3),axis=0)
cl_zonalMean_heightCoords_hg_A4 = np.nanmean(np.nanmean(cl_4D_heightCoords_hg_A4, axis=3),axis=0)

#Save using filename convention for the other script, "cloudFractionZonalMeanProfiles.py", that will plot this model with the others
np.save('npyFiles/zonalMeans_cl_PI_hg.npy', cl_zonalMean_heightCoords_hg_PI)
np.save('npyFiles/zonalMeans_cl_G1_hg.npy', cl_zonalMean_heightCoords_hg_G1)
np.save('npyFiles/zonalMeans_cl_A4_hg.npy', cl_zonalMean_heightCoords_hg_A4)


#####   CODE FOR TEST PLOTS BELOW THIS LINE   #####


##Load the zonal mean changes once they've been saved
cl_zonalMean_heightCoords_hg_PI = np.load('npyFiles/zonalMeans_cl_PI_hg.npy')
cl_zonalMean_heightCoords_hg_G1 = np.load('npyFiles/zonalMeans_cl_G1_hg.npy')
cl_zonalMean_heightCoords_hg_A4 = np.load('npyFiles/zonalMeans_cl_A4_hg.npy')



modelLats = np.load('npyFiles/modelLats.npy').item() #"cl is NOT on the "ta" grid, so use this version. 
modelLons = np.load('npyFiles/modelLons.npy').item() #This would make pressure coordinate transformation even harder...


def plotSingleSimContourfHeight(figNo, key, run, save=False):
    plt.figure(figNo)
    lat = modelLats[key]
    if(run == 'G1'):
        plotData = cl_zonalMean_heightCoords_hg_G1
        titleString = 'Cloud fraction, zonal mean, G1, years 11-50: \n'+gmf.modelNames[key]+' model'
    elif(run == 'piControl'):
        plotData = cl_zonalMean_heightCoords_hg_PI
        titleString = 'Cloud fraction, zonal mean, piControl, years 1-40: \n'+gmf.modelNames[key]+' model'
    else: #abrupt4xCO2 run
        plotData = cl_zonalMean_heightCoords_hg_A4
        titleString = 'Cloud fraction, zonal mean, abrupt4xCO2, years 11-50: \n'+gmf.modelNames[key]+' model'
    #Cloud fraction already in percent for this model. 
    print(len(lat))
    print(len(zLevels))
    print(np.shape(plotData))
    thePlot = plt.contourf(np.sin(lat*np.pi/180.), zLevels, plotData, levels=np.linspace(0.,50.,11), cmap=plt.cm.Blues_r)
    #plt.gca().invert_yaxis() #Is this necessary?
    plt.xlabel('Latitude (deg N)')
    plt.xticks(np.sin(np.array([-90, -60, -30, 0, 30, 60, 90])*np.pi/180.),  ['-90', '-60', '-30', '0', '30', '60', '90'])
    plt.ylabel('Height (m)')
    plt.yticks(np.linspace(0, 40000, 11))
    cbar = plt.colorbar(thePlot)
    cbar.ax.set_ylabel('Cloud fraction (%)')
    plt.title(titleString) 
    plt.gca().set_axis_bgcolor('k') 
    if(save):   
        plt.savefig('plots/individualModels/cl_zonalMean_contourf_'+run+'_hg_heightCoords.pdf')      
        
plotSingleSimContourfHeight(0, 'hg', 'G1', save=True)        
        
def plotDiffContourfHeight(figNo, key, run1, run2, save=False, clevs='default', ext='both'):
    plt.figure(figNo)
    lat = modelLats[key]
    titleString = 'Cloud fraction difference, zonal mean, '+run1 + ' - ' + run2 + ':\n'+gmf.modelNames[key]+' model'
    if(run1 == 'G1' and run2 == 'piControl'):
        plotData = cl_zonalMean_heightCoords_hg_G1 - cl_zonalMean_heightCoords_hg_PI 
    elif(run1 == 'G1' and run2 == 'abrupt4xCO2'):
        plotData = cl_zonalMean_heightCoords_hg_G1 - cl_zonalMean_heightCoords_hg_A4
    elif(run1 == 'abrupt4xCO2' and run2 == 'piControl'):
        plotData = cl_zonalMean_heightCoords_hg_A4 - cl_zonalMean_heightCoords_hg_PI
    else:
        print('Haven\'t programmed this combination of runs yet; exiting plot difference function')
        return
    if(clevs == 'default'): #Use the default contour levels, going from min to max of what is found'
        thePlot = plt.contourf(np.sin(lat*np.pi/180.), zLevels, plotData, cmap=plt.cm.RdBu, extend=ext)
        cbar = plt.colorbar(thePlot)
    else:   #Use specified contour levels
        thePlot = plt.contourf(np.sin(lat*np.pi/180.), zLevels, plotData, levels=clevs, cmap=plt.cm.RdBu, extend=ext)
        cbar = plt.colorbar(thePlot, ticks=np.linspace(min(clevs), max(clevs), 11))
    #plt.gca().invert_yaxis()
    plt.xlabel('Latitude (deg N)')
    plt.xticks(np.sin(np.array([-90, -60, -30, 0, 30, 60, 90])*np.pi/180.),  ['-90', '-60', '-30', '0', '30', '60', '90'])
    plt.ylabel('Height (m)')
    plt.yticks(np.linspace(0, 40000, 11))
    cbar.ax.set_ylabel('Cloud fraction difference (%)')
    plt.title(titleString)
    plt.gca().set_axis_bgcolor('k') 
    if(save):   
        plt.savefig('plots/individualModels/cl_zonalMean_contourf_'+run1+'-'+run2+'_hg_heightCoords.pdf')   
        
plotDiffContourfHeight(1, 'hg', 'G1', 'piControl', clevs=np.linspace(-3,3,12), save=True)
plotDiffContourfHeight(2, 'hg', 'abrupt4xCO2', 'piControl', clevs=np.linspace(-3,3,12), save=True)




#Might want to do above plots with height on log scale for better comparison with the pressure-coordinate plots for the other variables. 

#Now, try to get in pressure coordinates. 
#This will require loading the temperature data in pressure coordinates; regridding these to height coordinates using the hypsometric equation;
#Then using the temperatures to regrid the cloud fraction back to pressure coordinates again using the hypsometric equation. 
#Will this cause issues with NaNs? 
#Problem: unless I extrapolate at the ends, will lose the bottom pressure level (1000 hPa) when I do this, which will skew my low cloud stats. 
# Maybe should just use height definitions for low/middle/high clouds for this model. 


#Possible problem: if the heigtht to interpolate to happens to be the same as the original level, what happens? 
#Dividing by 0, but it would be multiplied by 0 too... what is 0 * np.nan? still nan
#However, I seem to be saved by the property of NumPy arrays that they return zero when dividing by 0, unlike normal Python integers--
#But this dooesn't work for floats, get "inf". And 0*inf returns nan, so this will disappear in the nanmean. 
#Will I have 3 slices that are totally empty over land? 
#0/0 for floats returns nan. 
#Could this have occurred for other models too? i.e. if one of the pressure levels wihtout terrain equaled one of the standard CMIP5 pressure levels? 
#Probably not, given dependence on surface pressure which is a float and varies in time. 
#I could get around this by just changing the few levels that matched...

#Okay, after changing to interpolate only to heights that aren't HadGEM default heights, cloud fraction plots look much more reasonable. 
#Still need to do vertical log scale though so it can be easily compared to the other models. 

#Actually vertical scale should be negative exponential (since p = p0*exp(-z/H))--use scale height of 8 km

def plotSingleSimContourfHeightLog(figNo, key, run, save=False):
    plt.figure(figNo)
    lat = modelLats[key]
    if(run == 'G1'):
        plotData = cl_zonalMean_heightCoords_hg_G1
        titleString = 'Cloud fraction, zonal mean, G1, years 11-50: \n'+gmf.modelNames[key]+' model'
    elif(run == 'piControl'):
        plotData = cl_zonalMean_heightCoords_hg_PI
        titleString = 'Cloud fraction, zonal mean, piControl, years 1-40: \n'+gmf.modelNames[key]+' model'
    else: #abrupt4xCO2 run
        plotData = cl_zonalMean_heightCoords_hg_A4
        titleString = 'Cloud fraction, zonal mean, abrupt4xCO2, years 11-50: \n'+gmf.modelNames[key]+' model'
    #Cloud fraction already in percent for this model. 
    print(len(lat))
    print(len(zLevels))
    print(np.shape(plotData))
    thePlot = plt.contourf(np.sin(lat*np.pi/180.), np.exp(-1./8000.*zLevels), plotData, levels=np.linspace(0.,50.,11), cmap=plt.cm.Blues_r)
    #plt.gca().invert_yaxis() #Is this necessary?
    #plt.gca().set_yscale('log') #This has problems with contourf
    plt.xlabel('Latitude (deg N)')
    plt.xticks(np.sin(np.array([-90, -60, -30, 0, 30, 60, 90])*np.pi/180.),  ['-90', '-60', '-30', '0', '30', '60', '90'])
    plt.ylabel('Height (m)')
    plt.yticks(np.exp(-1./8000.*np.array([0, 1500., 3250., 6500., 10000, 20000])), ['0', '1500', '3250', '6500', '10000', '20000'])
    plt.axis([-1., 1., np.exp(-1./8000.*0), np.exp(-1./8000.*20000)])
    cbar = plt.colorbar(thePlot)
    cbar.ax.set_ylabel('Cloud fraction (%)')
    plt.title(titleString) 
    plt.gca().set_axis_bgcolor('k') 
    if(save):   
        plt.savefig('plots/individualModels/cl_zonalMean_contourf_'+run+'_hg_heightCoords.pdf')      
        
plotSingleSimContourfHeightLog(3, 'hg', 'G1', save=True)        
        
def plotDiffContourfHeightLog(figNo, key, run1, run2, save=False, clevs='default', ext='both'):
    plt.figure(figNo)
    lat = modelLats[key]
    titleString = 'Cloud fraction difference, zonal mean, '+run1 + ' - ' + run2 + ':\n'+gmf.modelNames[key]+' model'
    if(run1 == 'G1' and run2 == 'piControl'):
        plotData = cl_zonalMean_heightCoords_hg_G1 - cl_zonalMean_heightCoords_hg_PI 
    elif(run1 == 'G1' and run2 == 'abrupt4xCO2'):
        plotData = cl_zonalMean_heightCoords_hg_G1 - cl_zonalMean_heightCoords_hg_A4
    elif(run1 == 'abrupt4xCO2' and run2 == 'piControl'):
        plotData = cl_zonalMean_heightCoords_hg_A4 - cl_zonalMean_heightCoords_hg_PI
    else:
        print('Haven\'t programmed this combination of runs yet; exiting plot difference function')
        return
    if(clevs == 'default'): #Use the default contour levels, going from min to max of what is found'
        thePlot = plt.contourf(np.sin(lat*np.pi/180.), np.exp(-1./8000.*zLevels), plotData, cmap=plt.cm.RdBu, extend=ext)
        cbar = plt.colorbar(thePlot)
    else:   #Use specified contour levels
        thePlot = plt.contourf(np.sin(lat*np.pi/180.), np.exp(-1./8000.*zLevels), plotData, levels=clevs, cmap=plt.cm.RdBu, extend=ext)
        cbar = plt.colorbar(thePlot, ticks=np.linspace(min(clevs), max(clevs), 11))
    #plt.gca().invert_yaxis()
    #plt.gca().set_yscale('log')
    plt.xlabel('Latitude (deg N)')
    plt.xticks(np.sin(np.array([-90, -60, -30, 0, 30, 60, 90])*np.pi/180.),  ['-90', '-60', '-30', '0', '30', '60', '90'])
    plt.ylabel('Height (m)')
    plt.yticks(np.exp(-1./8000.*np.array([0, 1500., 3250., 6500., 10000, 20000])), ['0', '1500', '3250', '6500', '10000', '20000'])
    plt.axis([-1., 1., np.exp(-1./8000.*0), np.exp(-1./8000.*20000)])
    cbar.ax.set_ylabel('Cloud fraction difference (%)')
    plt.title(titleString)
    plt.gca().set_axis_bgcolor('k') 
    if(save):   
        plt.savefig('plots/individualModels/cl_zonalMean_contourf_'+run1+'-'+run2+'_hg_heightCoords.pdf')   
        
plotDiffContourfHeightLog(4, 'hg', 'G1', 'piControl', clevs=np.linspace(-3,3,12), save=True)
plotDiffContourfHeightLog(5, 'hg', 'abrupt4xCO2', 'piControl', clevs=np.linspace(-10,10,12), save=True)




plt.show()
